@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 2 November 95
 --
---  Version $Revision: 1.30 $ from $Date: 2000/02/28 06:28:59 $
---
 --  Copyright (c) [1995..1999] Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -40,22 +38,14 @@
 module State (-- the PreCST monad
 	      --
 	      PreCST,					   -- reexport ABSTRACT
-	      nop, yield, (+>=), (+>), fixCST,             -- reexport
 	      throwExc, fatal, catchExc, fatalsHandledBy,  -- reexport lifted
 	      readCST, writeCST, transCST, run, runCST, 
-	      StateTrans.MVar, {-StateTrans.MArr, -}		   -- reexport
-	      newMV, readMV, assignMV, {-newMA, readMA,	   -- reexport lifted
-	      writeMA, StateTrans.boundsMA, -}		   -- reexport lifted
 	      --
 	      -- reexport compiler I/O
 	      --
 	      module CIO,
 	      liftIO,
 	      --
-	      -- identification
-	      --
-	      getId,
-	      --
 	      -- error management
 	      --
 	      raise, raiseWarning, raiseError, raiseFatal, showErrors,
@@ -70,23 +60,16 @@ module State (-- the PreCST monad
 	      getNameSupply)
 where
 
-import Ix
-import Monad       (when)
+import Control.Monad (when)
+import Data.List     (sort)
 
-import BaseVersion (version, copyright, disclaimer)
-import Config	   (errorLimit)
-import Common      (Position, errorCodeFatal)
-import Utils	   (sort)
+import Position    (Position)
 import UNames      (NameSupply,
 	            rootSupply, splitSupply)
-import StateTrans  (STB,
-		    readBase, transBase, runSTB)
+import StateTrans  (readBase, transBase, runSTB)
 import qualified
-       StateTrans  (interleave, throwExc, fatal, catchExc, fatalsHandledBy, 
-		    MVar, {-MArr,-}
-		    newMV, readMV, assignMV{-, newMA, readMA, writeMA, boundsMA-})
+       StateTrans  (interleave, throwExc, fatal, catchExc, fatalsHandledBy)
 import StateBase   (PreCST(..), ErrorState(..), BaseState(..),
-		    nop, yield, (+>=), (+>), fixCST,
 		    unpackCST, readCST, writeCST, transCST,
 		    liftIO)
 import CIO
@@ -100,10 +83,8 @@ import Errors      (ErrorLvl(..), Error, makeError, errorLvl, showError)
 --
 -- * it gets the version information and the initial extra state as arguments
 --
-initialBaseState        :: (String, String, String) -> e -> BaseState e
-initialBaseState vcd es  = BaseState {
-		             idTKBS     = (version, copyright, disclaimer),
-			     idBS       = vcd,
+initialBaseState   :: e -> BaseState e
+initialBaseState es = BaseState {
 			     errorsBS   = initialErrorState, 
 			     suppliesBS = splitSupply rootSupply,
 			     extraBS    = es
@@ -119,14 +100,14 @@ initialBaseState vcd es  = BaseState {
 -- * fatals errors are explicitly caught and reported (instead of letting them
 --   through to the runtime system)
 --
-run            :: (String, String, String) -> e -> PreCST e () a -> IO a
-run vcd es cst  = runSTB m (initialBaseState vcd es) ()
+run       :: e -> PreCST e () a -> IO a
+run es cst = runSTB m (initialBaseState es) ()
   where
     m = unpackCST (
 	  cst
 	  `fatalsHandledBy` \err ->
 	    putStrCIO ("Uncaught fatal error: " ++ show err)	>>
-	    exitWithCIO (ExitFailure errorCodeFatal)
+	    exitWithCIO (ExitFailure 1)
 	)
 
 -- run a PreCST in the context of another PreCST (EXPORTED)
@@ -184,42 +165,6 @@ fatalsHandledBy m h  = CST $ StateTrans.fatalsHandledBy m' h'
 		         m' = unpackCST m
 			 h' = unpackCST . h
 
--- mutable variables
--- -----------------
-
--- lifted mutable variable functions (EXPORTED)
---
-
-newMV :: a -> PreCST e s (StateTrans.MVar a)
-newMV  = CST . StateTrans.newMV
-
-readMV :: StateTrans.MVar a -> PreCST e s a
-readMV  = CST . StateTrans.readMV
-
-assignMV     :: StateTrans.MVar a -> a -> PreCST e s ()
-assignMV m a  = CST $ StateTrans.assignMV m a
-
-{- not used in c2hs:
-newMA     :: Ix i => (i, i) -> a -> PreCST e s (StateTrans.MArr i a)
-newMA b a  = CST $ StateTrans.newMA b a
-
-readMA      :: Ix i => StateTrans.MArr i a -> i -> PreCST e s a
-readMA  m i  = CST $ StateTrans.readMA m i
-
-writeMA       :: Ix i => StateTrans.MArr i a -> i -> a -> PreCST e s ()
-writeMA m i a  = CST $ StateTrans.writeMA m i a
- -}
-
-
--- read identification
--- -------------------
-
--- read identification information (EXPORT)
---
-getId :: PreCST e s (String, String, String)
-getId  = CST $ 
-         readBase (idBS)
-
 
 -- manipulating the error state
 -- ----------------------------
@@ -285,6 +230,8 @@ raise0 err  = do
 		  fatal ("Error limit of " ++ show errorLimit 
 			 ++ " errors has been reached.\n" ++ errmsgs)
   where
+    errorLimit = 20
+
     doRaise    :: BaseState e -> (BaseState e, Int)
     doRaise bs  = let
 		    lvl			       = errorLvl err
