@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 7 November 97
 --
---  Version $Revision: 1.12 $
---
 --  Copyright (C) [1997..1999] Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -41,7 +39,6 @@
 --
 --  * The following state components are maintained:
 --
---    + idBS (triple of strings)        -- version, copyright, and disclaimer
 --    + errorsBS (type `ErrorState')    -- keeps track of raised errors 
 --    + namesBS (type `NameSupply')     -- provides unique names
 --    + extraBS (generic type)		-- extra compiler-dependent state 
@@ -52,23 +49,16 @@
 --
 
 module StateBase (PreCST(..), ErrorState(..), BaseState(..),
-		  nop, yield, (+>=), (+>), fixCST,
 		  unpackCST, readCST, writeCST, transCST, liftIO)
 where
 
-import Common     (Position, errorCodeFatal)
 import UNames     (NameSupply)
-import StateTrans (STB, 
-		   fixSTB, readGeneric, writeGeneric, transGeneric, readBase,
-		   transBase)
+import StateTrans (STB, readGeneric, writeGeneric, transGeneric)
 import qualified  
        StateTrans (liftIO)
 import Errors     (ErrorLvl(..), Error)
 
 
-infixr 1 +>=, +>
-
-
 -- state used in the whole compiler
 -- --------------------------------
 
@@ -83,8 +73,6 @@ data ErrorState = ErrorState ErrorLvl    -- worst error level that was raised
 -- base state (EXPORTED)
 --
 data BaseState e = BaseState {
-		     idTKBS     :: (String, String, String),  -- toolkit id
-		     idBS       :: (String, String, String),  -- compiler id
 		     errorsBS   :: ErrorState, 
 		     suppliesBS :: [NameSupply],
 		     extraBS    :: e			      -- extra state
@@ -98,7 +86,6 @@ newtype PreCST e s a = CST (STB (BaseState e) s a)
 instance Monad (PreCST e s) where
   return = yield
   (>>=)  = (+>=)
-  (>>)   = (+>)
 
 
 -- unwrapper coercion function (EXPORTED)
@@ -120,21 +107,6 @@ yield a  = CST $ return a
 (+>=)   :: PreCST e s a -> (a -> PreCST e s b) -> PreCST e s b
 m +>= k  = CST $ unpackCST m >>= (\a -> unpackCST (k a))
 
--- bind dropping the result of the first state transfomer
---
-(+>)   :: PreCST e s a -> PreCST e s b -> PreCST e s b
-k +> m  = k +>= const m
-
--- unit with no result
---
-nop :: PreCST e s ()
-nop  = yield ()
-
--- fixpoint combinator in the monad (EXPORTED)
---
-fixCST   :: (a -> PreCST e s a) -> PreCST e s a
-fixCST m  = CST $ fixSTB (unpackCST . m)
-
 
 -- generic state manipulation
 -- --------------------------
