@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 3 March 95
 --
---  Version $Revision: 1.20 $ from $Date: 1999/11/30 17:08:21 $
---
 --  Copyright (C) [1995..1999] Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -63,7 +61,7 @@
 
 module StateTrans (-- the monad and the generic operations
 		   --
-		   STB, fixSTB,
+		   STB,
 		   --
 		   -- monad specific operations
 		   --
@@ -72,32 +70,9 @@ module StateTrans (-- the monad and the generic operations
 		   --
 		   -- exception handling and fatal errors
 		   --
-		   throwExc, fatal, catchExc, fatalsHandledBy, 
-		   --
-		   -- mutable variables and arrays
-		   --
-		   MVar, newMV, readMV, assignMV)
-		   -- Not used in c2hs:
-		   -- MArr, newMA, readMA, writeMA, boundsMA)
+		   throwExc, fatal, catchExc, fatalsHandledBy)
 where
 
--- standard libraries
-import Ix         (Ix)
-import Monad      (liftM)
-import System.IO  (fixIO)
-import Data.IORef (IORef, newIORef, readIORef, writeIORef)
-{-
-import IOExts (IOArray, newIOArray, boundsIOArray, readIOArray,
-		      writeIOArray)
- -}
-
--- CTK libraries
-import Common (assert)
-import Errors (interr)
-
-
-infixr 1 +>=, +>
-
 
 -- BEWARE! You enter monad country. Read any of Wadler's or 
 -- Launchbury/Peyton-Jones' texts before entering. Otherwise,
@@ -126,7 +101,6 @@ newtype STB bs gs a = STB (bs -> gs -> IO (bs, gs, Either (String, String) a))
 instance Monad (STB bs gs) where
   return = yield
   (>>=)  = (+>=)
-  (>>)   = (+>)
 
 -- the monad's unit
 --
@@ -149,31 +123,6 @@ m +>= k  = let
 				    in
 				    k' bs' gs'                   -- cont
 
--- bind dropping the result of the first state transfomer
---
-(+>)   :: STB bs gs a -> STB bs gs b -> STB bs gs b
-k +> m  = k +>= const m
-
--- fixpoint combinator in the monad
---
-fixSTB   :: (a -> STB bs gs a) -> STB bs gs a
---
--- builds on the fixpoint combinator embedded within the IO monad; the
--- future overall result wrapped into a closure with the function extracting
--- the user-level result component is used to build the cycle
---
-fixSTB m  = STB $ \bs gs 
-		  -> fixIO (\future -> let 
-					 STB m' = m (extractResult future) 
-				       in 
-				       m' bs gs)
-            where
-	      extractResult (_, _, Right r) = r
-	      extractResult (_, _, Left _ ) = interr "StateTrans: fixSTB: \
-						     \Tried to access result \
-						     \of unsuccessful \
-						     \recursive computation!"
-
 
 -- generic state manipulation
 -- --------------------------
@@ -352,33 +301,3 @@ fatalsHandledBy m handler  =
 			       STB handler' = handler err
 			     in
 			     handler' bs gs)
-
-
--- list mutable variables and arrays stuff into `STB'; all (EXPORTED)
--- ------------------------------------------------------------------
-
-type MVar a   = IORef a
--- type MArr i a = IOArray i a  -- not used in c2hs
-
-newMV   :: a -> STB bs gs (MVar a)
-newMV x  = liftIO (newIORef x)
-
-readMV    :: MVar a -> STB bs gs a
-readMV mv  = liftIO (readIORef mv)
-
-assignMV      :: MVar a -> a -> STB bs gs ()
-assignMV mv x  = liftIO (writeIORef mv x)
-
-{- not used in c2hs:
-newMA        :: Ix i => (i, i) -> a -> STB bs gs (MArr i a)
-newMA bnds x  = liftIO (newIOArray bnds x)
-
-boundsMA :: Ix i => IOArray i a -> (i, i)
-boundsMA  = boundsIOArray
-
-readMA      :: Ix i => MArr i a -> i -> STB bs gs a
-readMA ma i  = liftIO (readIOArray ma i)
-
-writeMA        :: Ix i => MArr i a -> i -> a -> STB bs gs ()
-writeMA ma i x  = liftIO (writeIOArray ma i x)
- -}
