@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 12 August 99
 --
---  Version $Revision: 1.9 $ from $Date: 2003/02/12 09:41:02 $
---
 --  Copyright (c) 1999 Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -35,10 +33,6 @@ module C (-- interface to KL for all non-KL modules
 	  -- stuff from `Common' (reexported)
 	  --
 	  Pos(posOf), 
-	  --
-	  -- reexported from `FiniteMaps'
-	  --
-	  FiniteMap,
 	  --	      
 	  -- structure tree
 	  --
@@ -47,7 +41,7 @@ module C (-- interface to KL for all non-KL modules
 	  -- attributed structure tree with operations (reexported from
 	  -- `CAttrs')
 	  --
-	  AttrC, getCHeader, 
+	  AttrC,
 	  CObj(..), CTag(..), CDef(..), lookupDefObjC, lookupDefTagC,
 	  getDefOfIdentC,
 	  --
@@ -55,10 +49,6 @@ module C (-- interface to KL for all non-KL modules
 	  --
 	  module CTrav,
 	  --
-	  -- support for pretty printing C abstract trees
-	  --
-	  module CPretty,
-	  --
 	  loadAttrC,		-- locally defined
 	  --
 	  -- misc. reexported stuff
@@ -70,22 +60,19 @@ module C (-- interface to KL for all non-KL modules
 	  csuffix, hsuffix, isuffix)
 where
 
-import Common     (Position, Pos(posOf))
-import FiniteMaps (FiniteMap)
-import Idents	  (Ident, lexemeToIdent)
+import Position   (Position(..), Pos(posOf))
+import Idents	  (Ident)
 import Attributes (Attrs, Attr(..))
 
-import C2HSState  (CST, IOMode(..),
-		   readCST, transCST, runCST, nop,
-		   readFileCIO, writeFileCIO, openFileCIO, hCloseCIO,
+import C2HSState  (CST,
+		   readFileCIO,
 		   fatal, errorsPresent, showErrors,
 		   Traces(..), putTraceStr)
 import CAST
 import CParser    (parseC)
-import CPretty
-import CAttrs	  (AttrC, attrC, getCHeader, 
-		   CObj(..), CTag(..), CDef(..), lookupDefObjC, lookupDefTagC,
-		   getDefOfIdentC)
+import CPretty    () -- just Show instances
+import CAttrs	  (AttrC, CObj(..), CTag(..), CDef(..),
+		   lookupDefObjC, lookupDefTagC, getDefOfIdentC)
 import CNames     (nameAnalysis)
 import CTrav
 
@@ -93,9 +80,9 @@ import CTrav
 -- suffix for files containing C (EXPORTED)
 --
 csuffix, hsuffix, isuffix :: String
-csuffix  = ".c"
-hsuffix  = ".h"
-isuffix  = ".i"
+csuffix  = "c"
+hsuffix  = "h"
+isuffix  = "i"
 
 -- given a file name (with suffix), parse that file as a C header and do the
 -- static analysis (collect defined names) (EXPORTED)
@@ -114,8 +101,7 @@ loadAttrC fname  = do
 		     -- parse
 		     --
 		     traceInfoParse
-		     rawHeader <- parseC contents (fname, 1, 1)
-		     let header = attrC rawHeader
+		     header <- parseC contents (Position fname 1 1)
 
 		     -- name analysis
 		     --
