@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 12 August 99
 --
---  Version $Revision: 1.8 $ from $Date: 2004/10/08 22:32:46 $
---
 --  Copyright (c) [1999..2001] Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -56,7 +54,7 @@
 
 module CAttrs (-- attributed C
 	       --
-	       AttrC, attrC, getCHeader, enterNewRangeC, enterNewObjRangeC,
+	       AttrC, emptyAttrC, enterNewRangeC, enterNewObjRangeC,
 	       leaveRangeC, leaveObjRangeC, addDefObjC, lookupDefObjC,
 	       lookupDefObjCShadow, addDefTagC, lookupDefTagC,
 	       lookupDefTagCShadow, applyPrefix, getDefOfIdentC,
@@ -68,11 +66,10 @@ module CAttrs (-- attributed C
 	       CObj(..), CTag(..), CDef(..))
 where
 
-import Char	  (toUpper)
-import List       (isPrefixOf)
-import Maybe	  (mapMaybe)
+import Data.Char  (toUpper)
+import Data.Maybe (mapMaybe)
 
-import Common     (Position, Pos(posOf), nopos, dontCarePos, builtinPos)
+import Position   (Pos(posOf), nopos, dontCarePos, builtinPos)
 import Errors     (interr)
 import Idents	  (Ident, getIdentAttrs, identToLexeme, onlyPosIdent)
 import Attributes (Attr(..), AttrTable, getAttr, setAttr, updAttr,
@@ -86,32 +83,25 @@ import CAST
 -- attributed C structure tree
 -- ---------------------------
 
--- C unit together with the attributes relevant to the outside world
+-- attributes relevant to the outside world gathjered from a C unit
 -- (EXPORTED ABSTRACT)
 --
 data AttrC = AttrC {
-	        headerAC  :: CHeader,		-- raw header
 		defObjsAC :: CObjNS,		-- defined objects
 		defTagsAC :: CTagNS,		-- defined tags
 		shadowsAC :: CShadowNS,		-- shadow definitions (prefix)
 		defsAC    :: CDefTable		-- ident-def associations
 	      }
 
--- make an attribute structure tree from a raw one (EXPORTED)
+-- empty headder attribute set (EXPORTED)
 --
-attrC        :: CHeader -> AttrC
-attrC header  = AttrC {
-		    headerAC  = header, 
-		    defObjsAC = cObjNS,
-		    defTagsAC = cTagNS,
-		    shadowsAC = cShadowNS,
-		    defsAC    = cDefTable
-		  }
-
--- extract the raw structure tree from an attributes one (EXPORTED)
---
-getCHeader :: AttrC -> CHeader
-getCHeader  = headerAC
+emptyAttrC :: AttrC
+emptyAttrC  = AttrC {
+	     defObjsAC = cObjNS,
+	     defTagsAC = cTagNS,
+	     shadowsAC = cShadowNS,
+	     defsAC    = cDefTable
+	   }
 
 
 -- the name space operations
