@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 16 October 99
 --
---  Version $Revision: 1.8 $ from $Date: 2004/06/11 07:10:16 $
---
 --  Copyright (c) 1999 Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -37,17 +35,16 @@
 module CNames (nameAnalysis)
 where
 
-import Monad	 (when, mapM_)
+import Control.Monad (mapM_)
 
-import Common	 (Position, posOf)
-import Utils     (mapMaybeM_)
+import Position  (Position, posOf)
 import Idents	 (Ident, identToLexeme)
 
-import C2HSState (CST, nop)
+import C2HSState (CST)
 import CAST
-import CAttrs    (AttrC, CObj(..), CTag(..), CDef(..))
+import CAttrs    (AttrC, emptyAttrC, CObj(..), CTag(..), CDef(..))
 import CBuiltin  (builtinTypeNames)
-import CTrav     (CT, getCHeaderCT, runCT, enter, enterObjs, leave, leaveObjs,
+import CTrav     (CT, runCT, enterObjs, leaveObjs,
 		  ifCTExc, raiseErrorCTExc, defObj, findTypeObj, findValueObj,
 		  defTag, refersToDef, isTypedef) 
 
@@ -61,10 +58,10 @@ type NA a = CT () a
 
 -- name analysis of C header files (EXPORTED)
 --
-nameAnalysis    :: AttrC -> CST s AttrC
-nameAnalysis ac  = do
-		     (ac', _) <- runCT naCHeader ac ()
-		     return ac'
+nameAnalysis         :: CHeader -> CST s AttrC
+nameAnalysis headder  = do
+			  (ac', _) <- runCT (naCHeader headder) emptyAttrC ()
+			  return ac'
 
 
 -- name analyis traversal
@@ -74,16 +71,15 @@ nameAnalysis ac  = do
 --
 -- * in case of an error, back off the current declaration
 --
-naCHeader :: NA ()
-naCHeader  = do
+naCHeader :: CHeader -> NA ()
+naCHeader (CHeader decls _) = do
 	       -- establish definitions for builtins
 	       --
 	       mapM_ (uncurry defObjOrErr) builtinTypeNames
 	       --
 	       -- analyse the header
 	       --
-	       CHeader decls _ <- getCHeaderCT
-	       mapM_ (\decl -> naCExtDecl decl `ifCTExc` nop) decls
+	       mapM_ (\decl -> naCExtDecl decl `ifCTExc` return ()) decls
 
 -- Processing of toplevel declarations
 --
@@ -111,7 +107,7 @@ naCDecl decl@(CDecl specs decls _) =
 
 naCDeclSpec :: CDeclSpec -> NA ()
 naCDeclSpec (CTypeSpec tspec) = naCTypeSpec tspec
-naCDeclSpec _		      = nop
+naCDeclSpec _		      = return ()
 
 naCTypeSpec :: CTypeSpec -> NA ()
 naCTypeSpec (CSUType   su   _) = naCStructUnion (StructUnionCT su) su
@@ -119,7 +115,7 @@ naCTypeSpec (CEnumType enum _) = naCEnum (EnumCT enum) enum
 naCTypeSpec (CTypeDef  ide  _) = do
 				   (obj, _) <- findTypeObj ide False
 				   ide `refersToDef` ObjCD obj
-naCTypeSpec _		       = nop
+naCTypeSpec _		       = return ()
 
 naCStructUnion :: CTag -> CStructUnion -> NA ()
 naCStructUnion tag (CStruct _ oide decls _) =
@@ -144,7 +140,7 @@ naCDeclr obj (CVarDeclr oide _) =
   mapMaybeM_ (`defObjOrErr` obj) oide
 naCDeclr obj (CPtrDeclr _ declr _   ) =
   naCDeclr obj declr
-naCDeclr obj (CArrDeclr declr oexpr _   ) =
+naCDeclr obj (CArrDeclr declr _ oexpr _   ) =
   do
     naCDeclr obj declr
     mapMaybeM_ naCExpr oexpr
@@ -157,12 +153,12 @@ naCDeclr obj (CFunDeclr declr decls _ _ ) =
 
 naCInit :: CInit -> NA ()
 naCInit (CInitExpr expr  _) = naCExpr expr
-naCInit (CInitList inits _) = mapM_ naCInit inits
+naCInit (CInitList inits _) = mapM_ (naCInit . snd) inits
 
 naCExpr :: CExpr -> NA ()
 naCExpr (CComma      exprs             _) = mapM_ naCExpr exprs
 naCExpr (CAssign     _ expr1 expr2     _) = naCExpr expr1 >> naCExpr expr2
-naCExpr (CCond       expr1 expr2 expr3 _) = naCExpr expr1 >> naCExpr expr2
+naCExpr (CCond       expr1 expr2 expr3 _) = naCExpr expr1 >> mapMaybeM_ naCExpr expr2
 					    >> naCExpr expr3
 naCExpr (CBinary     _ expr1 expr2     _) = naCExpr expr1 >> naCExpr expr2
 naCExpr (CCast       decl expr	       _) = naCDecl decl >> naCExpr expr
@@ -177,7 +173,8 @@ naCExpr (CMember      expr ide _       _) = naCExpr expr
 naCExpr (CVar	      ide	       _) = do
 					     (obj, _) <- findValueObj ide False
 					     ide `refersToDef` ObjCD obj
-naCExpr (CConst	      _		       _) = nop
+naCExpr (CConst	      _		       _) = return ()
+naCExpr (CCompoundLit _ inits          _) = mapM_ (naCInit . snd) inits
 
 
 -- auxilliary functions
@@ -189,7 +186,7 @@ defTagOrErr           :: Ident -> CTag -> NA ()
 ide `defTagOrErr` tag  = do
 			   otag <- ide `defTag` tag
 			   case otag of
-			     Nothing   -> nop
+			     Nothing   -> return ()
 			     Just tag' -> declaredTwiceErr ide (posOf tag')
 
 -- associate an object with a referring identifier
@@ -198,7 +195,13 @@ ide `defTagOrErr` tag  = do
 --   consistency of the declarations should be checked
 --
 defObjOrErr           :: Ident -> CObj -> NA ()
-ide `defObjOrErr` obj  = ide `defObj` obj >> nop
+ide `defObjOrErr` obj  = ide `defObj` obj >> return ()
+
+-- maps some monad operation into a `Maybe', discarding the result
+--
+mapMaybeM_ :: Monad m => (a -> m b) -> Maybe a -> m ()
+mapMaybeM_ m Nothing   =        return ()
+mapMaybeM_ m (Just a)  = m a >> return ()
 
 
 -- error messages
