@@ -3,8 +3,6 @@
 --  Author : Manuel M. T. Chakravarty
 --  Created: 16 October 99
 --
---  Version $Revision: 1.13 $ from $Date: 2001/08/26 08:34:24 $
---
 --  Copyright (c) [1999..2001] Manuel M. T. Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -61,7 +59,7 @@
 --   we have a user error, but currently an internal error is raised
 --
 
-module CTrav (CT, readCT, transCT, getCHeaderCT, runCT, throwCTExc, ifCTExc,
+module CTrav (CT, readCT, transCT, runCT, throwCTExc, ifCTExc,
 	      raiseErrorCTExc,
 	      enter, enterObjs, leave, leaveObjs, defObj, findObj,
 	      findObjShadow, defTag, findTag, findTagShadow,
@@ -80,19 +78,19 @@ module CTrav (CT, readCT, transCT, getCHeaderCT, runCT, throwCTExc, ifCTExc,
 	      lookupDeclOrTag)
 where
 
-import List       (find)
-import Maybe	  (fromMaybe)
-import Monad	  (liftM)
+import Data.List         (find)
+import Control.Monad     (liftM)
+import Control.Exception (assert)
 
-import Common	  (Position, Pos(..), nopos, assert)
+import Position   (Position, Pos(..), nopos)
 import Errors	  (interr)
-import Idents	  (Ident, dumpIdent, identToLexeme)
+import Idents	  (Ident, identToLexeme)
 import Attributes (Attr(..), newAttrsOnlyPos)
 
-import C2HSState  (CST, nop, readCST, transCST, runCST, raiseError, catchExc,
+import C2HSState  (CST, readCST, transCST, runCST, raiseError, catchExc,
 		   throwExc, Traces(..), putTraceStr)
 import CAST
-import CAttrs     (AttrC, getCHeader, enterNewRangeC, enterNewObjRangeC,
+import CAttrs     (AttrC, enterNewRangeC, enterNewObjRangeC,
 		   leaveRangeC, leaveObjRangeC, addDefObjC, lookupDefObjC,
 		   lookupDefObjCShadow, addDefTagC, lookupDefTagC,
 		   lookupDefTagCShadow, applyPrefix, getDefOfIdentC,
@@ -140,11 +138,6 @@ transCT trans  = transCST $ \(ac, s) -> let
 -- usage of a traversal monad
 --
 
--- get the raw C header from the monad (EXPORTED)
---
-getCHeaderCT :: CT s CHeader
-getCHeaderCT  = readAttrCCT getCHeader
-
 -- execute a traversal monad (EXPORTED)
 --
 -- * given a traversal monad, an attribute structure tree, and a user
@@ -312,11 +305,8 @@ applyPrefixToNameSpaces prefix  =
 getDefOf     :: Ident -> CT s CDef
 getDefOf ide  = do
 		  def <- readAttrCCT $ \ac -> getDefOfIdentC ac ide
-		  assert (not . isUndef $ def) err $
+		  assert (not . isUndef $ def) $
 		    return def
-		where
-		  err = "CTrav.getDefOf: Undefined object!\n\
-			\Object associated with " ++ dumpIdent ide
 
 -- set the definition of an identifier (EXPORTED) 
 --
@@ -530,12 +520,12 @@ tagName tag  =
 -- * as far as parameter passing is concerned, arrays are also pointer
 --
 isPtrDeclr                                 :: CDeclr -> Bool
-isPtrDeclr (CPtrDeclr _ (CVarDeclr _ _) _)  = True
-isPtrDeclr (CPtrDeclr _ declr           _)  = isPtrDeclr declr
-isPtrDeclr (CArrDeclr (CVarDeclr _ _) _ _)  = True
-isPtrDeclr (CArrDeclr declr _           _)  = isPtrDeclr declr
-isPtrDeclr (CFunDeclr declr _ _         _)  = isPtrDeclr declr
-isPtrDeclr _                                = False
+isPtrDeclr (CPtrDeclr _ (CVarDeclr _ _)   _)  = True
+isPtrDeclr (CPtrDeclr _ declr             _)  = isPtrDeclr declr
+isPtrDeclr (CArrDeclr (CVarDeclr _ _) _ _ _)  = True
+isPtrDeclr (CArrDeclr declr _ _           _)  = isPtrDeclr declr
+isPtrDeclr (CFunDeclr declr _ _           _)  = isPtrDeclr declr
+isPtrDeclr _                                  = False
 
 -- drops the first pointer level from the given declarator (EXPORTED)
 --
@@ -546,19 +536,16 @@ isPtrDeclr _                                = False
 --	  into an anonymous declarator and also change its attributes
 --
 dropPtrDeclr                                          :: CDeclr -> CDeclr
-dropPtrDeclr (CPtrDeclr qs declr@(CVarDeclr _ _) ats)  
-  | length qs == 1				       = declr
-  | otherwise					       =
-    CPtrDeclr (init qs) declr ats
-dropPtrDeclr (CPtrDeclr qs  declr                 ats) = 
+dropPtrDeclr (CPtrDeclr qs declr@(CVarDeclr _ _) ats)  = declr
+dropPtrDeclr (CPtrDeclr qs  declr                ats)  = 
   let declr' = dropPtrDeclr declr
   in
   CPtrDeclr qs declr' ats
-dropPtrDeclr (CArrDeclr declr@(CVarDeclr _ _) _ _)     = declr
-dropPtrDeclr (CArrDeclr declr                 e ats)   = 
+dropPtrDeclr (CArrDeclr declr@(CVarDeclr _ _) _ _ _)   = declr
+dropPtrDeclr (CArrDeclr declr                tq e ats) =
   let declr' = dropPtrDeclr declr
   in
-  CArrDeclr declr' e ats
+  CArrDeclr declr' tq e ats
 dropPtrDeclr (CFunDeclr declr args vari         ats)   =
   let declr' = dropPtrDeclr declr
   in
@@ -580,7 +567,7 @@ isPtrDecl _				    =
 --
 isFunDeclr                                   :: CDeclr -> Bool
 isFunDeclr (CPtrDeclr _ declr             _)  = isFunDeclr declr
-isFunDeclr (CArrDeclr declr _             _)  = isFunDeclr declr
+isFunDeclr (CArrDeclr declr _ _           _)  = isFunDeclr declr
 isFunDeclr (CFunDeclr (CVarDeclr _ _) _ _ _)  = True
 isFunDeclr (CFunDeclr declr _ _           _)  = isFunDeclr declr
 isFunDeclr _				      = False
@@ -613,10 +600,10 @@ funResultAndArgs (CDecl specs [(Just declr, _, _)] _) =
       let (args, declr', variadic) = funArgs declr
       in
       (args, CPtrDeclr qs declr' at, variadic)
-    funArgs (CArrDeclr declr oe                          at) =
+    funArgs (CArrDeclr declr tqs oe                      at) =
       let (args, declr', variadic) = funArgs declr
       in
-      (args, CArrDeclr declr' oe at, variadic)
+      (args, CArrDeclr declr' tqs oe at, variadic)
     funArgs (CFunDeclr declr args var                    at) =
       let (args, declr', variadic) = funArgs declr
       in
@@ -695,7 +682,7 @@ checkForOneCUName decl@(CDecl specs _ _)  =
 	case declaredDeclr decl of
 	  Nothing			-> n
           Just (CVarDeclr _ _         ) -> n
-	  Just (CPtrDeclr [_] (CVarDeclr _ _) _) -> Nothing
+	  Just (CPtrDeclr _ (CVarDeclr _ _) _) -> Nothing
     _                                  -> Nothing
 
 -- smart lookup
@@ -807,7 +794,7 @@ extractAlias decl@(CDecl specs _ _) ind =
       case declaredDeclr decl of
 	Nothing				       -> Just (ide', ind)
 	Just (CVarDeclr _ _                  ) -> Just (ide', ind)
-	Just (CPtrDeclr [_] (CVarDeclr _ _) _)
+	Just (CPtrDeclr _ (CVarDeclr _ _) _)
 	  | ind                                -> Just (ide', False)
 	  | otherwise			       -> Nothing
 	_                                      -> Nothing
@@ -836,22 +823,22 @@ extractStruct pos (StructUnionCT su)  =
 declrName                          :: CDeclr -> Maybe Ident
 declrName (CVarDeclr oide       _)  = oide
 declrName (CPtrDeclr _ declr    _)  = declrName declr
-declrName (CArrDeclr declr    _ _)  = declrName declr
+declrName (CArrDeclr declr  _ _ _)  = declrName declr
 declrName (CFunDeclr declr  _ _ _)  = declrName declr
 
 -- raise an error if the given declarator does not declare a C function or if
 -- the function is supposed to return an array (the latter is illegal in C)
 --
 assertFunDeclr :: Position -> CDeclr -> CT s ()
-assertFunDeclr pos (CArrDeclr (CFunDeclr (CVarDeclr _ _) _ _ _) _ _) = 
+assertFunDeclr pos (CArrDeclr (CFunDeclr (CVarDeclr _ _) _ _ _) _ _ _) =
   illegalFunResultErr pos
-assertFunDeclr pos            (CFunDeclr (CVarDeclr _ _) _ _ _)      = 
-  nop -- everything is ok
-assertFunDeclr pos            (CFunDeclr declr           _ _ _)      =
+assertFunDeclr pos            (CFunDeclr (CVarDeclr _ _) _ _ _)        =
+  return () -- everything is ok
+assertFunDeclr pos            (CFunDeclr declr           _ _ _)        =
   assertFunDeclr pos declr
-assertFunDeclr pos            (CPtrDeclr _ declr             _)      =
+assertFunDeclr pos            (CPtrDeclr _ declr             _)        =
   assertFunDeclr pos declr
-assertFunDeclr pos            (CArrDeclr declr             _ _)      =
+assertFunDeclr pos            (CArrDeclr declr           _ _ _)        =
   assertFunDeclr pos declr
 assertFunDeclr pos _					             = 
   funExpectedErr pos
@@ -861,7 +848,7 @@ assertFunDeclr pos _					             =
 --
 assertIfEnumThenFull                          :: CTag -> CT s ()
 assertIfEnumThenFull (EnumCT (CEnum _ [] at))  = enumForwardErr (posOf at)
-assertIfEnumThenFull _			       = nop
+assertIfEnumThenFull _			       = return ()
 
 -- trace for this module
 --
