@@ -3,8 +3,6 @@
 --  Author : Manuel M T Chakravarty
 --  Created: 16 August 99
 --
---  Version $Revision: 1.27 $ from $Date: 2005/03/14 00:26:58 $
---
 --  Copyright (c) [1999..2005] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -65,7 +63,7 @@
 --  prefix   -> `prefix' `=' string
 --  deriving -> `deriving' `(' ident_1 `,' ... `,' ident_n `)'
 --  parms    -> [verbhs `=>'] `{' parm_1 `,' ... `,' parm_n `}' `->' parm
---  parm     -> [ident_1 [`*' | `-']] verbhs [`&'] [ident_2 [`*' | `-']]
+--  parm     -> [ident_1 [`*' | `-']] verbhs [`&'] [ident_2 [`*'] [`-']]
 --  apath    -> ident
 --	      | `*' apath
 --	      | apath `.' ident
@@ -92,23 +90,25 @@ module CHS (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
 	    CHSChangeCase(..), CHSParm(..), CHSArg(..), CHSAccess(..),
 	    CHSAPath(..), CHSPtrType(..), 
 	    loadCHS, dumpCHS, hssuffix, chssuffix, loadCHI, dumpCHI, chisuffix,
-	    showCHSParm)
+	    showCHSParm, apathToIdent)
 where 
 
 -- standard libraries
-import Char	 (isSpace, toUpper, toLower)
-import List	 (intersperse)
-import Monad	 (when)
+import Data.Char (isSpace, toUpper, toLower)
+import Data.List (intersperse)
+import Control.Monad (when)
+import System.FilePath ((<.>), (</>))
 
 -- Compiler Toolkit
-import Common    (Position, Pos(posOf), nopos, isBuiltinPos)
+import Position  (Position(..), Pos(posOf), nopos, isBuiltinPos)
 import Errors	 (interr)
 import Idents    (Ident, identToLexeme, onlyPosIdent)
 
 -- C->Haskell
-import C2HSState (CST, doesFileExistCIO, readFileCIO, writeFileCIO, getId, 
+import C2HSState (CST, doesFileExistCIO, readFileCIO, writeFileCIO,
 		  getSwitch, chiPathSB, catchExc, throwExc, raiseError, 
 		  fatal, errorsPresent, showErrors, Traces(..), putTraceStr) 
+import Version    (version)
 
 -- friends
 import CHSLexer  (CHSToken(..), lexCHS)
@@ -141,6 +141,7 @@ data CHSFrag = CHSVerb String			-- Haskell code
 	     | CHSHook CHSHook			-- binding hook
 	     | CHSCPP  String			-- pre-processor directive
 		       Position
+             | CHSLine Position                 -- line pragma
 	     | CHSC    String			-- C code
 		       Position
 	     | CHSCond [(Ident,			-- C variable repr. condition
@@ -151,6 +152,7 @@ instance Pos CHSFrag where
   posOf (CHSVerb _ pos ) = pos
   posOf (CHSHook hook  ) = posOf hook
   posOf (CHSCPP  _ pos ) = pos
+  posOf (CHSLine   pos ) = pos
   posOf (CHSC    _ pos ) = pos
   posOf (CHSCond alts _) = case alts of
 			     (_, frag:_):_ -> posOf frag
@@ -177,12 +179,12 @@ data CHSHook = CHSImport  Bool			-- qualified?
 			  Position
 	     | CHSCall    Bool			-- is a pure function?
 			  Bool			-- is unsafe?
-			  Ident			-- C function
+			  CHSAPath              -- C function
 			  (Maybe Ident)		-- Haskell name
 			  Position
 	     | CHSFun     Bool			-- is a pure function?
 			  Bool			-- is unsafe?
-			  Ident			-- C function
+			  CHSAPath              -- C function
 			  (Maybe Ident)		-- Haskell name
 			  (Maybe String)	-- type context
 			  [CHSParm]		-- argument marshalling
@@ -271,6 +273,7 @@ data CHSParm = CHSParm (Maybe (Ident, CHSArg))	-- "in" marshaller
 data CHSArg = CHSValArg				-- plain value argument
 	    | CHSIOArg				-- reference argument
 	    | CHSVoidArg			-- no argument
+            | CHSIOVoidArg                      -- drops argument, but in monad
 	    deriving (Eq)
 
 -- structure member access types (EXPORTED)
@@ -286,6 +289,11 @@ data CHSAPath = CHSRoot  Ident			-- root of access path
 	      | CHSRef	 CHSAPath Ident		-- member referencing
 	      deriving (Eq)
 
+instance Pos CHSAPath where
+    posOf (CHSRoot ide)    = posOf ide
+    posOf (CHSDeref _ pos) = pos
+    posOf (CHSRef _ ide)   = posOf ide
+
 -- pointer options (EXPORTED)
 --
 
@@ -315,8 +323,8 @@ instance Read CHSPtrType where
 -- ------------------------
 
 hssuffix, chssuffix :: String
-hssuffix  = ".hs"
-chssuffix = ".chs"
+hssuffix  = "hs"
+chssuffix = "chs"
 
 -- load a CHS module (EXPORTED)
 --
@@ -327,7 +335,7 @@ chssuffix = ".chs"
 --
 loadCHS       :: FilePath -> CST s (CHSModule, String)
 loadCHS fname  = do
-		   let fullname = fname ++ chssuffix
+		   let fullname = fname <.> chssuffix
 
 		   -- read file
 		   --
@@ -337,7 +345,7 @@ loadCHS fname  = do
 		   -- parse
 		   --
 		   traceInfoParse
-		   mod <- parseCHSModule (fullname, 1, 1) contents
+		   mod <- parseCHSModule (Position fullname 1 1) contents
 
 		   -- check for errors and finalize
 		   --
@@ -379,8 +387,7 @@ dumpCHS fname mod pureHaskell  =
     let (suffix, kind) = if pureHaskell
 			 then (hssuffix , "(Haskell)")
 			 else (chssuffix, "(C->HS binding)")
-    (version, _, _) <- getId
-    writeFileCIO (fname ++ suffix) (contents version kind)
+    writeFileCIO (fname <.> suffix) (contents version kind)
   where
     contents version kind = 
       "-- GENERATED by " ++ version ++ " " ++ kind ++ "\n\
@@ -411,7 +418,7 @@ showCHSModule (CHSModule frags) pureHaskell  =
     showFrags _      _     []                           = id
     showFrags pureHs state (CHSVerb s      pos : frags) = 
       let
-	(fname, line, _) = pos
+	(Position fname line _) = pos
 	generated	 = isBuiltinPos pos
 	emitNow		 = state == Emit || 
 			   (state == Wait && not (null s) && head s == '\n')
@@ -433,6 +440,8 @@ showCHSModule (CHSModule frags) pureHaskell  =
       . showString s
 --      . showChar '\n'
       . showFrags False Emit frags
+    showFrags pureHs _     (CHSLine s          : frags) =
+        showFrags pureHs Emit frags
     showFrags False  _     (CHSC    s    _     : frags) =
         showString "\n#c"
       . showString s
@@ -473,12 +482,12 @@ showCHSHook (CHSCall isPure isUns ide oalias _) =
     showString "call "
   . (if isPure then showString "pure " else id)
   . (if isUns then showString "unsafe " else id)
-  . showIdAlias ide oalias
+  . showApAlias ide oalias
 showCHSHook (CHSFun isPure isUns ide oalias octxt parms parm _) =   
     showString "fun "
   . (if isPure then showString "pure " else id)
   . (if isUns then showString "unsafe " else id)
-  . showIdAlias ide oalias
+  . showApAlias ide oalias
   . (case octxt of
        Nothing      -> showChar ' '
        Just ctxtStr -> showString ctxtStr . showString " => ")
@@ -531,6 +540,13 @@ showIdAlias ide oalias  =
        Nothing  -> id
        Just ide -> showString " as " . showCHSIdent ide)
 
+showApAlias            :: CHSAPath -> Maybe Ident -> ShowS
+showApAlias apath oalias  =
+    showCHSAPath apath
+  . (case oalias of
+       Nothing  -> id
+       Just ide -> showString " as " . showCHSIdent ide)
+
 showCHSParm                                                :: CHSParm -> ShowS
 showCHSParm (CHSParm oimMarsh hsTyStr twoCVals oomMarsh _)  =
     showOMarsh oimMarsh
@@ -543,9 +559,10 @@ showCHSParm (CHSParm oimMarsh hsTyStr twoCVals oomMarsh _)  =
     showOMarsh Nothing               = id
     showOMarsh (Just (ide, argKind)) =   showCHSIdent ide
 				       . (case argKind of
-					   CHSValArg  -> id
-					   CHSIOArg   -> showString "*"
-					   CHSVoidArg -> showString "-")
+					   CHSValArg    -> id
+					   CHSIOArg     -> showString "*"
+					   CHSVoidArg   -> showString "-"
+                                           CHSIOVoidArg -> showString "*-")
     --
     showHsVerb str = showChar '`' . showString str . showChar '\''
 
@@ -592,7 +609,7 @@ showCHSIdent  = showString . identToLexeme
 -- ------------------------
 
 chisuffix :: String
-chisuffix  = ".chi"
+chisuffix  = "chi"
 
 versionPrefix :: String
 versionPrefix  = "C->Haskell Interface Version "
@@ -611,10 +628,10 @@ loadCHI fname  = do
 		   -- search for .chi files
 		   --
 		   paths <- getSwitch chiPathSB
-		   let fullnames = [path ++ '/':fname ++ chisuffix | 
+		   let fullnames = [path </> fname <.> chisuffix | 
 				    path <- paths]
 		   fullname <- findFirst fullnames
-		     (fatal $ fname++chisuffix++" not found in:\n"++
+		     (fatal $ fname <.> chisuffix ++ " not found in:\n"++
 			      unlines paths)
 		   -- read file
 		   --
@@ -637,7 +654,6 @@ loadCHI fname  = do
 				       Nothing     -> errorCHICorrupt fname
 				       Just majMin -> return majMin
 		     
-		   (version, _, _) <- getId
 		   let Just (myMajor, myMinor) = majorMinor version
 		   when (major /= myMajor || minor /= myMinor) $
 		     errorCHIVersion fname 
@@ -671,8 +687,7 @@ loadCHI fname  = do
 dumpCHI                :: String -> String -> CST s ()
 dumpCHI fname contents  =
   do
-    (version, _, _) <- getId
-    writeFileCIO (fname ++ chisuffix) $
+    writeFileCIO (fname <.> chisuffix) $
       versionPrefix ++ version ++ "\n" ++ contents
 
 -- extract major and minor number from a version string
@@ -735,6 +750,9 @@ parseFrags toks  = do
     parseFrags0 (CHSTokCPP     pos s:toks) = do
 					       frags <- parseFrags toks
 					       return $ CHSCPP s pos : frags
+    parseFrags0 (CHSTokLine    pos  :toks) = do
+					       frags <- parseFrags toks
+					       return $ CHSLine pos : frags
     parseFrags0 (CHSTokC       pos s:toks) = parseC       pos s      toks 
     parseFrags0 (CHSTokImport  pos  :toks) = parseImport  pos        toks
     parseFrags0 (CHSTokContext pos  :toks) = parseContext pos        toks
@@ -846,20 +864,20 @@ parseCall pos toks  =
   do
     (isPure  , toks'   ) <- parseIsPure          toks
     (isUnsafe, toks''  ) <- parseIsUnsafe        toks'
-    (ide     , toks''' ) <- parseIdent           toks''
-    (oalias  , toks'''') <- parseOptAs ide False toks'''
+    (apath   , toks''' ) <- parsePath            toks''
+    (oalias  , toks'''') <- parseOptAs (apathToIdent apath) False toks'''
     toks'''''		 <- parseEndHook	 toks''''
     frags                <- parseFrags		 toks'''''
     return $ 
-      CHSHook (CHSCall isPure isUnsafe ide (norm ide oalias) pos) : frags
+      CHSHook (CHSCall isPure isUnsafe apath (normAP apath oalias) pos) : frags
 
 parseFun          :: Position -> [CHSToken] -> CST s [CHSFrag]
 parseFun pos toks  = 
   do
     (isPure  , toks' ) <- parseIsPure          toks
     (isUnsafe, toks'2) <- parseIsUnsafe        toks'
-    (ide     , toks'3) <- parseIdent           toks'2
-    (oalias  , toks'4) <- parseOptAs ide False toks'3
+    (apath   , toks'3) <- parsePath            toks'2
+    (oalias  , toks'4) <- parseOptAs (apathToIdent apath) False toks'3
     (octxt   , toks'5) <- parseOptContext      toks'4
     (parms   , toks'6) <- parseParms	       toks'5
     (parm    , toks'7) <- parseParm	       toks'6
@@ -867,7 +885,7 @@ parseFun pos toks  =
     frags              <- parseFrags	       toks'8
     return $ 
       CHSHook 
-        (CHSFun isPure isUnsafe ide (norm ide oalias) octxt parms parm pos) :
+        (CHSFun isPure isUnsafe apath (normAP apath oalias) octxt parms parm pos) :
       frags
   where
     parseOptContext (CHSTokHSVerb _ ctxt:CHSTokDArrow _:toks) =
@@ -901,6 +919,25 @@ parseIsUnsafe :: [CHSToken] -> CST s (Bool, [CHSToken])
 parseIsUnsafe (CHSTokUnsafe _:toks) = return (True , toks)
 parseIsUnsafe toks		    = return (False, toks)
 
+normAP :: CHSAPath -> Maybe Ident -> Maybe Ident
+normAP ide Nothing                                = Nothing
+normAP ide (Just ide') | apathToIdent ide == ide' = Nothing
+		       | otherwise                = Just ide'
+
+
+apathToIdent :: CHSAPath -> Ident
+apathToIdent (CHSRoot ide) =
+    let lowerFirst (c:cs) = toLower c : cs
+    in onlyPosIdent (posOf ide) (lowerFirst $ identToLexeme ide)
+apathToIdent (CHSDeref apath _) =
+    let ide = apathToIdent apath
+    in onlyPosIdent (posOf ide) (identToLexeme ide ++ "_")
+apathToIdent (CHSRef apath ide') =
+    let ide = apathToIdent apath
+        upperFirst (c:cs) = toLower c : cs
+        sel = upperFirst $ identToLexeme ide'
+    in onlyPosIdent (posOf ide) (identToLexeme ide ++ sel)
+
 norm :: Ident -> Maybe Ident -> Maybe Ident
 norm ide Nothing                   = Nothing
 norm ide (Just ide') | ide == ide' = Nothing
@@ -921,6 +958,8 @@ parseParm toks =
     return (CHSParm oimMarsh hsTyStr twoCVals oomMarsh pos, toks'3)
   where
     parseOptMarsh :: [CHSToken] -> CST s (Maybe (Ident, CHSArg), [CHSToken])
+    parseOptMarsh (CHSTokIdent _ ide:CHSTokStar _ :CHSTokMinus _:toks) = 
+      return (Just (ide, CHSIOVoidArg) , toks)
     parseOptMarsh (CHSTokIdent _ ide:CHSTokStar _ :toks) = 
       return (Just (ide, CHSIOArg) , toks)
     parseOptMarsh (CHSTokIdent _ ide:CHSTokMinus _:toks) = 
@@ -934,7 +973,8 @@ parseField :: Position -> CHSAccess -> [CHSToken] -> CST s [CHSFrag]
 parseField pos access toks =
   do
     (path, toks') <- parsePath  toks
-    frags         <- parseFrags toks'
+    toks''        <- parseEndHook toks'
+    frags         <- parseFrags toks''
     return $ CHSHook (CHSField access path pos) : frags
 
 parsePointer :: Position -> [CHSToken] -> CST s [CHSFrag]
@@ -1077,9 +1117,7 @@ parsePath' (CHSTokArrow pos:CHSTokIdent _ ide:toks) =
 parsePath' (CHSTokArrow _:toks) = 
   syntaxError toks
 parsePath' toks =
-  do
-    toks' <- parseEndHook toks
-    return (id, toks')
+    return (id,toks)
 
 parseTrans :: [CHSToken] -> CST s (CHSTrans, [CHSToken])
 parseTrans (CHSTokLBrace _:toks) =
