@@ -3,8 +3,6 @@
 --  Author : Manuel M T Chakravarty
 --  Derived: 18 February 2 (extracted from GenBind.hs)
 --
---  Version $Revision: 1.4 $ from $Date: 2003/10/19 10:46:10 $
---
 --  Copyright (c) [2002..2005] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -72,24 +70,23 @@ module GBMonad (
 ) where 
 
 -- standard libraries
-import Char	  (toUpper, toLower, isSpace)
-import List       (find)
-import Maybe	  (fromMaybe)
+import Data.Char  (toUpper, toLower, isSpace)
+import Data.List  (find)
+import Data.Maybe (fromMaybe)
 
 -- Compiler Toolkit
-import Common     (Position, Pos(posOf), nopos, builtinPos)
+import Position   (Position, Pos(posOf), nopos)
 import Errors	  (interr)
 import Idents     (Ident, identToLexeme, onlyPosIdent)
-import FiniteMaps (FiniteMap, zeroFM, addToFM, lookupFM, joinFM, toListFM,
-		   listToFM)
+import qualified Data.Map as Map (empty, insert, lookup, union, toList, fromList)
+import Data.Map   (Map)
 
 -- C -> Haskell
 import C	  (CT, readCT, transCT, raiseErrorCTExc)
 
 -- friends
-import CHS	  (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
-		   CHSChangeCase(..), CHSAccess(..), CHSAPath(..),
-		   CHSPtrType(..))
+import CHS	  (CHSFrag(..), CHSHook(..), CHSTrans(..),
+		   CHSChangeCase(..), CHSPtrType(..))
 
 
 -- translation tables
@@ -187,7 +184,7 @@ transTabToTransFun prefix (CHSTrans _2Case chgCase table) =
 --   for type arguments to parametrised pointer types, ie, it holds for `res'
 --   in `Int -> IO res', but not in `Int -> Ptr res'
 --
-type PointerMap = FiniteMap (Bool, Ident) (String, String)
+type PointerMap = Map (Bool, Ident) (String, String)
 
 -- map that maintains key information about some of the Haskell objects
 -- generated by c2hs
@@ -204,7 +201,7 @@ data HsObject    = Pointer {
 		     ptrHO	  :: Ident	   -- pointer
 		   }
                  deriving (Show, Read)
-type HsObjectMap = FiniteMap Ident HsObject
+type HsObjectMap = Map Ident HsObject
 
 {- FIXME: What a mess...
 instance Show HsObject where
@@ -252,8 +249,8 @@ initialGBState  = GBState {
 		    lib    = "",
 		    prefix = "",
 		    frags  = [],
-		    ptrmap = zeroFM,
-		    objmap = zeroFM
+		    ptrmap = Map.empty,
+		    objmap = Map.empty
 		  }
 
 -- set the dynamic library and library prefix
@@ -314,7 +311,7 @@ getDelayedCode  = readCT (map snd . frags)
 ptrMapsTo :: (Bool, Ident) -> (String, String) -> GB ()
 (isStar, cName) `ptrMapsTo` hsRepr =
   transCT (\state -> (state { 
-		        ptrmap = addToFM (isStar, cName) hsRepr (ptrmap state)
+		        ptrmap = Map.insert (isStar, cName) hsRepr (ptrmap state)
 		      }, ()))
 
 -- query the pointer map
@@ -322,14 +319,14 @@ ptrMapsTo :: (Bool, Ident) -> (String, String) -> GB ()
 queryPtr        :: (Bool, Ident) -> GB (Maybe (String, String))
 queryPtr pcName  = do
 		     fm <- readCT ptrmap
-		     return $ lookupFM fm pcName
+		     return $ Map.lookup pcName fm
 
 -- add an entry to the Haskell object map
 --
 objIs :: Ident -> HsObject -> GB ()
 hsName `objIs` obj =
   transCT (\state -> (state { 
-		        objmap = addToFM hsName obj (objmap state)
+		        objmap = Map.insert hsName obj (objmap state)
 		      }, ()))
 
 -- query the Haskell object map
@@ -337,7 +334,7 @@ hsName `objIs` obj =
 queryObj        :: Ident -> GB (Maybe HsObject)
 queryObj hsName  = do
 		     fm <- readCT objmap
-		     return $ lookupFM fm hsName
+		     return $ Map.lookup hsName fm
 
 -- query the Haskell object map for a class
 --
@@ -382,15 +379,15 @@ queryPointer hsName  = do
 mergeMaps     :: String -> GB ()
 mergeMaps str  =
   transCT (\state -> (state { 
-		        ptrmap = joinFM readPtrMap (ptrmap state),
-		        objmap = joinFM readObjMap (objmap state)
+		        ptrmap = Map.union readPtrMap (ptrmap state),
+		        objmap = Map.union readObjMap (objmap state)
 		      }, ()))
   where
     (ptrAssoc, objAssoc) = read str
-    readPtrMap           = listToFM [((isStar, onlyPosIdent nopos ide), repr)
-			            | ((isStar, ide), repr) <- ptrAssoc]
-    readObjMap           = listToFM [(onlyPosIdent nopos ide, obj)
-			            | (ide, obj)            <- objAssoc]
+    readPtrMap           = Map.fromList [((isStar, onlyPosIdent nopos ide), repr)
+				        | ((isStar, ide), repr) <- ptrAssoc]
+    readObjMap           = Map.fromList [(onlyPosIdent nopos ide, obj)
+				        | (ide, obj)            <- objAssoc]
 
 -- convert the whole pointer and Haskell object maps into printable form
 --
@@ -399,9 +396,9 @@ dumpMaps  = do
 	      ptrFM <- readCT ptrmap
 	      objFM <- readCT objmap
 	      let dumpable = ([((isStar, identToLexeme ide), repr)
-			      | ((isStar, ide), repr) <- toListFM ptrFM],
+			      | ((isStar, ide), repr) <- Map.toList ptrFM],
 			      [(identToLexeme ide, obj)
-			      | (ide, obj)            <- toListFM objFM])
+			      | (ide, obj)            <- Map.toList objFM])
 	      return $ show dumpable
 
 
