@@ -1,4 +1,5 @@
 --  C->Haskell Compiler: binding generator
+--  vim:ts=8:noexpandtab
 --
 --  Copyright (c) [1999..2003] Manuel M T Chakravarty
 --
@@ -105,33 +106,29 @@ module GenBind (expandHooks)
 where 
 
 -- standard libraries
-import Char	  (toUpper, toLower, isSpace)
-import List       (deleteBy, intersperse, isPrefixOf)
-import Maybe	  (isNothing, isJust, fromJust, fromMaybe)
-import Monad	  (when, unless, liftM, mapAndUnzipM)
+import Data.Char     (toLower)
+import Data.List     (deleteBy, intersperse, find)
+import Data.Maybe    (isNothing, fromJust, fromMaybe)
+import Control.Monad (when, unless, liftM, mapAndUnzipM)
 
 -- Compiler Toolkit
-import Common     (Position, Pos(posOf), nopos, builtinPos)
-import Utils	  (lookupBy, mapMaybeM)
+import Position   (Position, Pos(posOf), nopos, builtinPos)
 import Errors	  (interr, todo)
 import Idents     (Ident, identToLexeme, onlyPosIdent)
 import Attributes (newAttrsOnlyPos)
 
 -- C->Haskell
 import C2HSConfig (PlatformSpec(..))
-import C2HSState  (CST, nop, errorsPresent, showErrors, fatal,
-		   SwitchBoard(..), Traces(..), putTraceStr, getSwitch,
-		   printCIO)
-import C	  (AttrC, CObj(..), CTag(..), lookupDefObjC, lookupDefTagC,
-		   CHeader(..), CExtDecl, CDecl(..), CDeclSpec(..),
-		   CStorageSpec(..), CTypeSpec(..), CTypeQual(..),
+import C2HSState  (CST, errorsPresent, showErrors, fatal,
+		   SwitchBoard(..), Traces(..), putTraceStr, getSwitch)
+import C	  (AttrC, CObj(..), CTag(..),
+		   CDecl(..), CDeclSpec(..), CTypeSpec(..), 
 		   CStructUnion(..), CStructTag(..), CEnum(..), CDeclr(..),
-		   CInit(..), CExpr(..), CAssignOp(..), CBinaryOp(..),
-		   CUnaryOp(..), CConst (..),
-		   CT, readCT, transCT, getCHeaderCT, runCT, ifCTExc,
+		   CExpr(..), CBinaryOp(..), CUnaryOp(..), CConst (..),
+		   runCT, ifCTExc,
 		   raiseErrorCTExc, findValueObj, findFunObj, findTag,
-		   findTypeObj, applyPrefixToNameSpaces, isTypedef,
-		   simplifyDecl, declrFromDecl, declrNamed, structMembers,
+		   applyPrefixToNameSpaces,
+		   simplifyDecl, declrNamed, structMembers,
 		   structName, tagName, declaredName , structFromDecl,
 		   funResultAndArgs, chaseDecl, findAndChaseDecl,
 		   checkForAlias, checkForOneAliasName, checkForOneCUName, 
@@ -140,14 +137,14 @@ import C	  (AttrC, CObj(..), CTag(..), lookupDefObjC, lookupDefTagC,
 		   refersToNewDef, CDef(..))
 
 -- friends
-import CHS	  (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
+import CHS	  (CHSModule(..), CHSFrag(..), CHSHook(..),
 		   CHSParm(..), CHSArg(..), CHSAccess(..), CHSAPath(..),
-		   CHSPtrType(..), showCHSParm) 
+		   CHSPtrType(..), showCHSParm, apathToIdent) 
 import CInfo      (CPrimType(..), size, alignment, getPlatform)
 import GBMonad    (TransFun, transTabToTransFun, HsObject(..), GB,
 		   initialGBState, setContext, getPrefix, 
 		   delayCode, getDelayedCode, ptrMapsTo, queryPtr, objIs,
-		   queryObj, queryClass, queryPointer, mergeMaps, dumpMaps)
+		   queryClass, queryPointer, mergeMaps, dumpMaps)
 
 
 -- default marshallers
@@ -237,6 +234,13 @@ isFloatHsType "Float"  = True
 isFloatHsType "Double" = True
 isFloatHsType _	       = False
 
+isVariadic :: ExtType -> Bool
+isVariadic (FunET s t)  = any isVariadic [s,t]
+isVariadic (IOET t)     = isVariadic t
+isVariadic (PtrET t)    = isVariadic t
+isVariadic (VarFunET _) = True
+isVariadic _            = False
+
 -- check for integral C types
 --
 -- * For marshalling purposes C char's are integral types (see also types
@@ -260,7 +264,7 @@ cFromBoolIde      = noPosIdent "cFromBool"
 cToBoolIde        = noPosIdent "cToBool"
 cIntConvIde       = noPosIdent "cIntConv"
 cFloatConvIde     = noPosIdent "cFloatConv"
-withIde           = noPosIdent "withObject"   -- FIXME: should be "with"
+withIde           = noPosIdent "with"
 withCStringIde    = noPosIdent "withCString"
 withCStringLenIde = noPosIdent "withCStringLenIntConv"
 withIntConvIde    = noPosIdent "withIntConv"
@@ -327,6 +331,7 @@ expandFrags = liftM concat . mapM expandFrag
 
 expandFrag :: CHSFrag -> GB [CHSFrag]
 expandFrag verb@(CHSVerb _ _     ) = return [verb]
+expandFrag line@(CHSLine _       ) = return [line]
 expandFrag      (CHSHook h       ) = 
   do
     code <- expandHook h
@@ -372,7 +377,7 @@ expandHook (CHSImport qual ide chi _) =
 expandHook (CHSContext olib oprefix _) =
   do
     setContext olib oprefix		      -- enter context information
-    mapMaybeM applyPrefixToNameSpaces oprefix -- use the prefix on name spaces
+    mapMaybeM_ applyPrefixToNameSpaces oprefix -- use the prefix on name spaces
     return ""
 expandHook (CHSType ide pos) =
   do
@@ -380,6 +385,7 @@ expandHook (CHSType ide pos) =
     decl <- findAndChaseDecl ide False True	-- no indirection, but shadows
     ty <- extractSimpleType False pos decl
     traceInfoDump decl ty
+    when (isVariadic ty) (variadicErr pos (posOf decl))
     return $ "(" ++ showExtType ty ++ ")"
   where
     traceInfoType         = traceGenBind "** Type hook:\n"
@@ -414,7 +420,7 @@ expandHook (CHSEnum cide oalias chsTrans oprefix derive _) =
     let trans = transTabToTransFun prefix chsTrans
 	hide  = identToLexeme . fromMaybe cide $ oalias
     enumDef enum hide trans (map identToLexeme derive)
-expandHook hook@(CHSCall isPure isUns ide oalias pos) =
+expandHook hook@(CHSCall isPure isUns (CHSRoot ide) oalias pos) =
   do
     traceEnter
     -- get the corresponding C declaration; raises error if not found or not a
@@ -430,7 +436,38 @@ expandHook hook@(CHSCall isPure isUns ide oalias pos) =
   where
     traceEnter = traceGenBind $ 
       "** Call hook for `" ++ identToLexeme ide ++ "':\n"
-expandHook hook@(CHSFun isPure isUns ide oalias ctxt parms parm pos) =
+expandHook hook@(CHSCall isPure isUns apath oalias pos) =
+  do
+    traceEnter
+    
+    (decl, offsets) <- accessPath apath
+    ptrTy <- extractSimpleType False pos decl
+    ty <- case ptrTy of
+        PtrET f@(FunET _ _) -> return f
+        _ -> funPtrExpectedErr pos
+        
+    traceValueType ty
+    set_get <- setGet pos CHSGet offsets ptrTy
+
+    -- get the corresponding C declaration; raises error if not found or not a
+    -- function; we use shadow identifiers, so the returned identifier is used 
+    -- afterwards instead of the original one
+    --
+    -- (ObjCO cdecl, ide) <- findFunObj ide True
+    let ideLexeme = identToLexeme $ apathToIdent apath
+	hsLexeme  = ideLexeme `maybe` identToLexeme $ oalias
+        -- cdecl'    = ide `simplifyDecl` cdecl
+        args      = concat [ " x" ++ show n | n <- [1..numArgs ty] ]
+
+    callImportDyn hook isPure isUns ideLexeme hsLexeme ty pos
+    return $ "(\\o" ++ args ++ " -> " ++ set_get ++ " o >>= \\f -> "
+             ++ hsLexeme ++ " f" ++ args ++ ")"
+  where
+    traceEnter = traceGenBind $ 
+      "** Indirect call hook for `" ++ identToLexeme (apathToIdent apath) ++ "':\n"
+    traceValueType et  = traceGenBind $ 
+      "Type of accessed value: " ++ showExtType et ++ "\n"
+expandHook hook@(CHSFun isPure isUns (CHSRoot ide) oalias ctxt parms parm pos) =
   do
     traceEnter
     -- get the corresponding C declaration; raises error if not found or not a
@@ -443,12 +480,54 @@ expandHook hook@(CHSFun isPure isUns ide oalias ctxt parms parm pos) =
 	fiLexeme  = hsLexeme ++ "'_"   -- *Urgh* - probably unqiue...
 	fiIde     = onlyPosIdent nopos fiLexeme
         cdecl'    = cide `simplifyDecl` cdecl
-	callHook  = CHSCall isPure isUns cide (Just fiIde) pos
+	callHook  = CHSCall isPure isUns (CHSRoot cide) (Just fiIde) pos
     callImport callHook isPure isUns (identToLexeme cide) fiLexeme cdecl' pos
-    funDef isPure hsLexeme fiLexeme cdecl' ctxt parms parm pos
+
+    extTy <- extractFunType pos cdecl' True
+    funDef isPure hsLexeme fiLexeme extTy ctxt parms parm Nothing pos
   where
     traceEnter = traceGenBind $ 
       "** Fun hook for `" ++ identToLexeme ide ++ "':\n"
+expandHook hook@(CHSFun isPure isUns apath oalias ctxt parms parm pos) =
+  do
+    traceEnter
+
+    (decl, offsets) <- accessPath apath
+    ptrTy <- extractSimpleType False pos decl
+    ty <- case ptrTy of
+        PtrET f@(FunET _ _) -> return f
+        _ -> funPtrExpectedErr pos
+        
+    traceValueType ty
+
+    -- get the corresponding C declaration; raises error if not found or not a
+    -- function; we use shadow identifiers, so the returned identifier is used 
+    -- afterwards instead of the original one
+    --
+    -- (ObjCO cdecl, cide) <- findFunObj ide True
+    let ideLexeme = identToLexeme $ apathToIdent apath
+	hsLexeme  = ideLexeme `maybe` identToLexeme $ oalias
+	fiLexeme  = hsLexeme ++ "'_"   -- *Urgh* - probably unqiue...
+	fiIde     = onlyPosIdent nopos fiLexeme
+        -- cdecl'    = cide `simplifyDecl` cdecl
+        args      = concat [ " x" ++ show n | n <- [1..numArgs ty] ]
+	callHook  = CHSCall isPure isUns apath (Just fiIde) pos
+    callImportDyn callHook isPure isUns ideLexeme fiLexeme ty pos
+
+    set_get <- setGet pos CHSGet offsets ptrTy
+    funDef isPure hsLexeme fiLexeme (FunET ptrTy $ purify ty)
+                  ctxt parms parm (Just set_get) pos
+  where
+    -- remove IO from the result type of a function ExtType.  necessary
+    -- due to an unexpected interaction with the way funDef works
+    purify (FunET a b) = FunET a (purify b)
+    purify (IOET b)    = b
+    purify a           = a
+
+    traceEnter = traceGenBind $ 
+      "** Fun hook for `" ++ identToLexeme (apathToIdent apath) ++ "':\n"
+    traceValueType et  = traceGenBind $ 
+      "Type of accessed value: " ++ showExtType et ++ "\n"
 expandHook (CHSField access path pos) =
   do
     traceInfoField
@@ -495,6 +574,8 @@ expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType emit
 			     et    <- extractPtrType cDecl
 			     traceInfoPtrType et
 			     let et' = adjustPtr isStar et
+                             when (isVariadic et')
+                                  (variadicErr pos (posOf cDecl))
 			     return (showExtType et', isFunExtType et')
 	    Just hsType -> return (identToLexeme hsType, False)
 	    -- FIXME: it is not possible to determine whether `hsType'
@@ -667,51 +748,95 @@ callImport hook isPure isUns ideLexeme hsLexeme cdecl pos =
     --
     extType <- extractFunType pos cdecl isPure
     header  <- getSwitch headerSB
+    when (isVariadic extType) (variadicErr pos (posOf cdecl))
     delayCode hook (foreignImport header ideLexeme hsLexeme isUns extType)
     traceFunType extType
   where
     traceFunType et = traceGenBind $ 
       "Imported function type: " ++ showExtType et ++ "\n"
 
+callImportDyn :: CHSHook -> Bool -> Bool -> String -> String -> ExtType
+              -> Position -> GB ()
+callImportDyn hook isPure isUns ideLexeme hsLexeme ty pos =
+  do
+    -- compute the external type from the declaration, and delay the foreign
+    -- export declaration
+    --
+    when (isVariadic ty) (variadicErr pos pos) -- FIXME? (posOf cdecl))
+    delayCode hook (foreignImportDyn ideLexeme hsLexeme isUns ty)
+    traceFunType ty
+  where
+    traceFunType et = traceGenBind $ 
+      "Imported function type: " ++ showExtType et ++ "\n"
+
 -- Haskell code for the foreign import declaration needed by a call hook
 --
 foreignImport :: String -> String -> String -> Bool -> ExtType -> String
 foreignImport header ident hsIdent isUnsafe ty  =
-  "foreign import ccall " ++ safety ++ " \"" ++ header ++ " " ++ ident ++ 
-  "\"\n  " ++ hsIdent ++ " :: " ++ showExtType ty ++ "\n"
+  "foreign import ccall " ++ safety ++ " " ++ show (header ++ " " ++ ident) ++
+  "\n  " ++ hsIdent ++ " :: " ++ showExtType ty ++ "\n"
+  where
+    safety = if isUnsafe then "unsafe" else "safe"
+
+-- Haskell code for the foreign import dynamic declaration needed by a call hook
+--
+foreignImportDyn :: String -> String -> Bool -> ExtType -> String
+foreignImportDyn ident hsIdent isUnsafe ty  =
+  "foreign import ccall " ++ safety ++ " \"dynamic\"\n  " ++
+    hsIdent ++ " :: FunPtr( " ++ showExtType ty ++ " ) -> " ++
+    showExtType ty ++ "\n"
   where
     safety = if isUnsafe then "unsafe" else "safe"
 
 -- produce a Haskell function definition for a fun hook
 --
+-- * FIXME: There's an ugly special case in here: to support dynamic fun hooks
+--   I had to add a special second marshaller for the first argument,
+--   which, if present, is inserted just before the function call.  This
+--   is probably not the most elegant solution, it's just the only one I
+--   can up with at the moment.  If present, this special marshaller is
+--   an io action (like 'peek' and unlike 'with'). -- US
+
 funDef :: Bool		     -- pure function?
        -> String	     -- name of the new Haskell function
        -> String	     -- Haskell name of the foreign imported C function
-       -> CDecl		     -- simplified declaration of the C function
+       -> ExtType            -- simplified declaration of the C function
        -> Maybe String	     -- type context of the new Haskell function
        -> [CHSParm]	     -- parameter marshalling description
        -> CHSParm	     -- result marshalling description 
+       -> Maybe String       -- optional additional marshaller for first arg
        -> Position	     -- source location of the hook
        -> GB String	     -- Haskell code in text form
-funDef isPure hsLexeme fiLexeme cdecl octxt parms parm pos =
+funDef isPure hsLexeme fiLexeme extTy octxt parms parm marsh2 pos =
   do
-    (parms', parm', isImpure) <- addDftMarshaller pos parms parm cdecl
+    (parms', parm', isImpure) <- addDftMarshaller pos parms parm extTy
+
     traceMarsh parms' parm' isImpure
     let 
       sig       = hsLexeme ++ " :: " ++ funTy parms' parm' ++ "\n"
       marshs    = [marshArg i parm | (i, parm) <- zip [1..] parms']
       funArgs   = [funArg   | (funArg, _, _, _, _)   <- marshs, funArg   /= ""]
       marshIns  = [marshIn  | (_, marshIn, _, _, _)  <- marshs]
-      callArgs  = [callArg  | (_, _, callArg, _, _)  <- marshs]
+      callArgs  = [callArg  | (_, _, cs, _, _)  <- marshs, callArg <- cs]
       marshOuts = [marshOut | (_, _, _, marshOut, _) <- marshs, marshOut /= ""]
       retArgs   = [retArg   | (_, _, _, _, retArg)   <- marshs, retArg   /= ""]
       funHead   = hsLexeme ++ join funArgs ++ " =\n" ++
 	          if isPure && isImpure then "  unsafePerformIO $\n" else ""
       call      = if isPure 
-		  then "  let {res = " ++ fiLexeme ++ join callArgs ++ "} in\n"
-		  else "  " ++ fiLexeme ++ join callArgs ++ " >>= \\res ->\n"
+		  then "  let {res = " ++ fiLexeme ++ joinCallArgs ++ "} in\n"
+		  else "  " ++ fiLexeme ++ joinCallArgs ++ " >>= \\res ->\n"
+      joinCallArgs = case marsh2 of
+      			Nothing -> join callArgs
+                        Just _  -> join ("b1'" : drop 1 callArgs)
+      mkMarsh2  = case marsh2 of
+      			Nothing -> ""
+			Just m  -> "  " ++ m ++ " " ++
+			           join (take 1 callArgs) ++
+				   " >>= \\b1' ->\n"
       marshRes  = case parm' of
 	            CHSParm _ _ twoCVal (Just (_    , CHSVoidArg)) _ -> ""
+	            CHSParm _ _ twoCVal (Just (omIde, CHSIOVoidArg)) _ ->
+	              "  " ++ identToLexeme omIde ++ " res >> \n"
 	            CHSParm _ _ twoCVal (Just (omIde, CHSIOArg  )) _ -> 
 	              "  " ++ identToLexeme omIde ++ " res >>= \\res' ->\n"
 	            CHSParm _ _ twoCVal (Just (omIde, CHSValArg )) _ -> 
@@ -719,10 +844,12 @@ funDef isPure hsLexeme fiLexeme cdecl octxt parms parm pos =
 		    CHSParm _ _ _       Nothing		           _ ->
 		      interr "GenBind.funDef: marshRes: no default?"
       retArgs'  = case parm' of
-	            CHSParm _ _ _ (Just (_, CHSVoidArg)) _ ->        retArgs
-	            _					   -> "res'":retArgs
+	            CHSParm _ _ _ (Just (_, CHSVoidArg))   _ ->        retArgs
+	            CHSParm _ _ _ (Just (_, CHSIOVoidArg)) _ ->        retArgs
+	            _					     -> "res'":retArgs
       ret       = "(" ++ concat (intersperse ", " retArgs') ++ ")"
       funBody   = joinLines marshIns  ++ 
+		  mkMarsh2            ++
 	          call                ++
 	          joinLines marshOuts ++ 
 		  marshRes            ++ 
@@ -758,7 +885,7 @@ funDef isPure hsLexeme fiLexeme cdecl octxt parms parm pos =
       where
         notVoid Nothing          = interr "GenBind.funDef: \
 					  \No default marshaller?"
-	notVoid (Just (_, kind)) = kind /= CHSVoidArg
+	notVoid (Just (_, kind)) = kind /= CHSVoidArg && kind /= CHSIOVoidArg
     --
     -- for an argument marshaller, generate all "in" and "out" marshalling
     -- code fragments
@@ -778,19 +905,20 @@ funDef isPure hsLexeme fiLexeme cdecl octxt parms parm pos =
 		     CHSIOArg   -> imApp ++ " $ \\" ++ inBndr ++ " -> "
 		     CHSValArg  -> "let {" ++ inBndr ++ " = " ++ 
 				   imApp ++ "} in "
-	callArg  = if twoCVal 
-		     then "" ++ a ++ "'1 " ++ a ++ "'2"
-		     else a ++ "'"
-	omApp	 = identToLexeme omIde ++ " " ++ callArg
+	callArgs = if twoCVal 
+		     then [a ++ "'1 ", a ++ "'2"]
+		     else [a ++ "'"]
+	omApp	 = identToLexeme omIde ++ join callArgs
 	outBndr  = a ++ "''"
         marshOut = case omArgKind of
-		     CHSVoidArg -> ""
-		     CHSIOArg   -> omApp ++ ">>= \\" ++ outBndr ++ " -> "
-		     CHSValArg  -> "let {" ++ outBndr ++ " = " ++ 
+		     CHSVoidArg   -> ""
+		     CHSIOVoidArg -> omApp ++ ">>"
+		     CHSIOArg     -> omApp ++ ">>= \\" ++ outBndr ++ " -> "
+		     CHSValArg    -> "let {" ++ outBndr ++ " = " ++ 
 				   omApp ++ "} in "
-	retArg   = if omArgKind == CHSVoidArg then "" else outBndr
+	retArg   = if omArgKind == CHSVoidArg || omArgKind == CHSIOVoidArg then "" else outBndr
       in
-      (funArg, marshIn, callArg, marshOut, retArg)
+      (funArg, marshIn, callArgs, marshOut, retArg)
     marshArg _ _ = interr "GenBind.funDef: Missing default?"
     --
     traceMarsh parms parm isImpure = traceGenBind $ 
@@ -806,10 +934,10 @@ funDef isPure hsLexeme fiLexeme cdecl octxt parms parm pos =
 
 -- add default marshallers for "in" and "out" marshalling
 --
-addDftMarshaller :: Position -> [CHSParm] -> CHSParm -> CDecl 
+addDftMarshaller :: Position -> [CHSParm] -> CHSParm -> ExtType
 		 -> GB ([CHSParm], CHSParm, Bool)
-addDftMarshaller pos parms parm cdecl = do
-  (resTy, argTys)     <- splitFunTy `liftM` extractFunType pos cdecl True
+addDftMarshaller pos parms parm extTy = do
+  let (resTy, argTys)  = splitFunTy extTy
   (parm' , isImpure1) <- checkResMarsh parm resTy
   (parms', isImpure2) <- addDft parms argTys
   return (parms', parm', isImpure1 || isImpure2)
@@ -926,7 +1054,7 @@ accessPath (CHSRef path ide) =				-- a.m
   where
     assertPrimDeclr ide (CDecl _ [declr] _) =
       case declr of
-        (Just (CVarDeclr _ _), _, _) -> nop
+        (Just (CVarDeclr _ _), _, _) -> return ()
 	_			     -> structExpectedErr ide
 accessPath (CHSDeref path pos) =			-- *a
   do
@@ -1049,6 +1177,7 @@ setGet pos access offsets ty =
     -- check that the type can be marshalled and compute extra operations for
     -- bitfields
     --
+    checkType (VarFunET  _    )          = variadicErr pos pos
     checkType (IOET      _    )          = interr "GenBind.setGet: Illegal \
 						  \type!"
     checkType (UnitET         )          = voidFieldErr pos
@@ -1198,6 +1327,7 @@ data ExtType = FunET     ExtType ExtType	-- function
 	     | DefinedET CDecl String		-- aliased type
 	     | PrimET    CPrimType		-- basic C type
 	     | UnitET				-- void
+             | VarFunET  ExtType                -- variadic function
 
 instance Eq ExtType where
   (FunET     t1 t2) == (FunET     t1' t2') = t1 == t1' && t2 == t2'
@@ -1205,6 +1335,7 @@ instance Eq ExtType where
   (PtrET     t    ) == (PtrET     t'     ) = t == t'
   (DefinedET _  s ) == (DefinedET _   s' ) = s == s'
   (PrimET    t    ) == (PrimET    t'     ) = t == t'
+  (VarFunET  t    ) == (VarFunET  t'     ) = t == t'
   UnitET	    == UnitET		   = True
 
 -- composite C type
@@ -1215,9 +1346,14 @@ data CompType = ExtType  ExtType		-- external type
 -- check whether an external type denotes a function type
 --
 isFunExtType             :: ExtType -> Bool
-isFunExtType (FunET _ _)  = True
-isFunExtType (IOET  _  )  = True
-isFunExtType _            = False
+isFunExtType (FunET    _ _) = True
+isFunExtType (VarFunET _  ) = True
+isFunExtType (IOET     _  ) = True
+isFunExtType _              = False
+
+numArgs                  :: ExtType -> Int
+numArgs (FunET _ f) = 1 + numArgs f
+numArgs _           = 0
 
 -- pretty print an external type
 --
@@ -1229,6 +1365,7 @@ showExtType                        :: ExtType -> String
 showExtType (FunET UnitET res)      = showExtType res
 showExtType (FunET arg res)	    = "(" ++ showExtType arg ++ " -> " 
 				      ++ showExtType res ++ ")"
+showExtType (VarFunET res)    	    = "( ... -> " ++ showExtType res ++ ")"
 showExtType (IOET t)		    = "(IO " ++ showExtType t ++ ")"
 showExtType (PtrET t)	            = let ptrCon = if isFunExtType t 
 						   then "FunPtr" else "Ptr"
@@ -1275,15 +1412,16 @@ extractFunType pos cdecl isPure  =
     -- the result
     --
     let (args, resultDecl, variadic) = funResultAndArgs cdecl
-    when variadic $
-      variadicErr pos cpos
     preResultType <- extractSimpleType True pos resultDecl
     --
     -- we can now add the `IO' monad if this is no pure function 
     --
-    let resultType = if isPure 
-		     then      preResultType 
-		     else IOET preResultType
+    let protoResultType = if isPure
+                          then      preResultType
+                          else IOET preResultType
+    let resultType = if variadic 
+                     then VarFunET protoResultType
+                     else          protoResultType
     --
     -- compute function arguments and create a function type (a function
     -- prototype with `void' as its single argument declares a nullary
@@ -1291,8 +1429,6 @@ extractFunType pos cdecl isPure  =
     --
     argTypes <- mapM (extractSimpleType False pos) args
     return $ foldr FunET resultType argTypes
-  where
-    cpos = posOf cdecl
 
 -- compute a non-struct/union type from the given declaration 
 --
@@ -1584,6 +1720,16 @@ addBitSize (BitSize o1 b1) (BitSize o2 b2)  = BitSize (o1 + o2 + overflow) rest
     bitsPerBitfield  = size CIntPT * 8
     (overflow, rest) = (b1 + b2) `divMod` bitsPerBitfield
 
+-- multiply a bit size by a constant (gives size of an array)
+--
+-- * not sure if this makes sense if the number of bits is non-zero.
+--
+scaleBitSize                  :: Int -> BitSize -> BitSize
+scaleBitSize n (BitSize o1 b1) = BitSize (n * o1 + overflow) rest
+  where
+    bitsPerBitfield  = size CIntPT * 8
+    (overflow, rest) = (n * b1) `divMod` bitsPerBitfield
+
 -- pad any storage unit that is partially used by a bitfield
 --
 padBits               :: BitSize -> Int
@@ -1639,17 +1785,41 @@ sizeAlignOfStructPad decls tag =
 -- compute the size and alignment constraint of a given C declaration
 --
 sizeAlignOf       :: CDecl -> GB (BitSize, Int)
+sizeAlignOfSingle :: CDecl -> GB (BitSize, Int)
 --
 -- * we make use of the assertion that `extractCompType' can only return a
 --   `DefinedET' when the declaration is a pointer declaration
+-- * for arrays, alignment is the same as for the base type and the size
+--   is the size of the base type multiplied by the number of elements.
+--   FIXME: I'm not sure whether anything of this is guaranteed by ISO C
+--   and I have no idea what happens when an array-of-bitfield is
+--   declared.  At this time I don't care.  -- U.S. 05/2006
 --
-sizeAlignOf cdecl  = 
+sizeAlignOf (CDecl dclspec
+                   [(Just (CArrDeclr declr _ (Just lexpr) _), init, expr)]
+		   attr) =
+  do
+    (bitsize, align) <- sizeAlignOf (CDecl dclspec
+                                           [(Just declr, init, expr)]
+					   attr)
+    IntResult length <- evalConstCExpr lexpr
+    return (fromIntegral length `scaleBitSize` bitsize, align)
+sizeAlignOf (CDecl _ [(Just (CArrDeclr cdecl _ Nothing _), _, _)] _) =
+    interr "GenBind.sizeAlignOf: array of undeclared size."
+sizeAlignOf cdecl =
+    sizeAlignOfSingle cdecl
+
+
+sizeAlignOfSingle cdecl  = 
   do
     ct <- extractCompType False False cdecl
     case ct of
       ExtType (FunET _ _        ) -> do
 				       align <-	alignment CFunPtrPT
 				       return (bitSize CFunPtrPT, align)
+      ExtType (VarFunET _       ) -> do
+				       align <-	alignment CFunPtrPT
+				       return (bitSize CFunPtrPT, align)
       ExtType (IOET  _          ) -> interr "GenBind.sizeof: Illegal IO type!"
       ExtType (PtrET t          ) 
         | isFunExtType t          -> do
@@ -1729,7 +1899,7 @@ evalConstCExpr (CComma _ at) =
   illegalConstExprErr (posOf at) "a comma expression"
 evalConstCExpr (CAssign _ _ _ at) =
   illegalConstExprErr (posOf at) "an assignment"
-evalConstCExpr (CCond b t e _) =
+evalConstCExpr (CCond b (Just t) e _) =
   do
     bv <- evalConstCExpr b
     case bv of
@@ -1902,6 +2072,17 @@ noPosIdent  = onlyPosIdent nopos
 traceGenBind :: String -> GB ()
 traceGenBind  = putTraceStr traceGenBindSW
 
+-- generic lookup
+--
+lookupBy      :: (a -> a -> Bool) -> a -> [(a, b)] -> Maybe b
+lookupBy eq x  = fmap snd . find (eq x . fst)
+
+-- maps some monad operation into a `Maybe', discarding the result
+--
+mapMaybeM_ :: Monad m => (a -> m b) -> Maybe a -> m ()
+mapMaybeM_ m Nothing   =        return ()
+mapMaybeM_ m (Just a)  = m a >> return ()
+
 
 -- error messages
 -- --------------
@@ -1970,6 +2151,12 @@ ptrExpectedErr pos  =
      "Attempt to dereference a non-pointer object or to use it in a `pointer' \
      \hook."]
 
+funPtrExpectedErr     :: Position -> GB a
+funPtrExpectedErr pos  =
+  raiseErrorCTExc pos
+    ["Expected a pointer-to-function object!",
+     "Attempt to use a non-pointer object in a `call' or `fun' hook."]
+
 illegalStablePtrErr     :: Position -> GB a
 illegalStablePtrErr pos  =
   raiseErrorCTExc pos
