@@ -3,8 +3,6 @@
 --  Author : Manuel M T Chakravarty
 --  Created: 6 March 99
 --
---  Version $Revision: 1.13 $ from $Date: 2004/10/17 08:31:08 $
---
 --  Copyright (c) [1999..2005] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -68,7 +66,6 @@ import C2HSConfig (PlatformSpec, defaultPlatformSpec)
 data SwitchBoard = SwitchBoard {
 		     cppOptsSB :: String,	-- cpp options
 		     cppSB     :: FilePath,	-- cpp executable
-		     dataSB    :: FilePath,	-- data directory
 		     keepSB    :: Bool,		-- keep intermediate file
 		     librarySB :: Bool,		-- copy library in
 		     tracesSB  :: Traces,	-- trace flags
@@ -85,7 +82,6 @@ initialSwitchBoard :: SwitchBoard
 initialSwitchBoard  = SwitchBoard {
 			cppOptsSB  = "",
 			cppSB      = "cpp",
-			dataSB	   = "",
 			keepSB	   = False,
 			librarySB  = False,
 		        tracesSB   = initialTraces,
