@@ -53,9 +53,6 @@
 --
 --        In the case of repeated occurences, the last takes effect.
 --
---  --data=DIR
---        Directory of data files.  (Usually only used by the wrapper script.)
---
 --  -d TYPE
 --  --dump=TYPE
 --        Dump intermediate representation:
@@ -111,7 +108,7 @@
 --
 --  -v,
 --  --version
---        Print (on standard error output) the version and copyright
+--        Print (on standard output) the version and copyright
 --	  information of the compiler (before doing anything else).
 --
 --- TODO ----------------------------------------------------------------------
@@ -121,41 +118,42 @@ module Main (main)
 where
 
 -- standard libraries
-import List	  (isPrefixOf, intersperse)
-import IO	  ()
-import Monad      (when, unless, mapM)
+import Data.List (isPrefixOf, intersperse, partition)
+import Control.Monad (when, unless)
+import Data.Version (showVersion)
 
 -- base libraries
-import Common     (errorCodeFatal)
 import System.Console.GetOpt     
 		  (ArgOrder(..), OptDescr(..), ArgDescr(..), usageInfo, getOpt)
-import FNameOps   (suffix, basename, dirname, stripSuffix, addPath)
-import Errors	  (interr)
+import qualified System.FilePath as FilePath 
+                  (takeExtension, dropExtension, takeDirectory, takeBaseName)
+import System.FilePath ((<.>), (</>))
+import StateBase  (liftIO)
 
 -- c2hs modules
-import C2HSState  (CST, nop, runC2HS, fatal, fatalsHandledBy, getId,
-		   ExitCode(..), stderr, IOMode(..), putStrCIO, putStrLnCIO,
+import C2HSState  (CST, runC2HS, fatal, fatalsHandledBy,
+		   ExitCode(..), stderr, putStrCIO, putStrLnCIO,
 		   hPutStrCIO, printCIO,
 		   hPutStrLnCIO, exitWithCIO, getArgsCIO, getProgNameCIO,
 		   ioeGetErrorString, ioeGetFileName, removeFileCIO,
-		   systemCIO, fileFindInCIO, mktempCIO, openFileCIO, hCloseCIO,
-		   readFileCIO, writeFileCIO,
+		   systemCIO, readFileCIO, writeFileCIO,
 		   SwitchBoard(..), Traces(..), setTraces,
 		   traceSet, setSwitch, getSwitch, putTraceStr)
-import C	  (AttrC, hsuffix, isuffix, loadAttrC)
-import CHS	  (CHSModule, loadCHS, dumpCHS, hssuffix, chssuffix, dumpCHI)
+import C	  (hsuffix, isuffix, loadAttrC)
+import CHS	  (loadCHS, dumpCHS, hssuffix, chssuffix, dumpCHI)
 import GenHeader  (genHeader)
 import GenBind	  (expandHooks)
-import Version    (version, copyright, disclaimer)
-import C2HSConfig (cpp, cppopts, datadir, libfname, tmpdir, PlatformSpec(..),
+import Version    (versnum, version, copyright, disclaimer)
+import C2HSConfig (cpp, cppopts, libfname, PlatformSpec(..),
 		   defaultPlatformSpec, platformSpecDB)
+import Paths_c2hs (getDataDir)
 
 
 -- wrapper running the compiler
 -- ============================
 
 main :: IO ()
-main  = runC2HS (version, copyright, disclaimer) compile
+main  = runC2HS compile
 
 
 -- option handling
@@ -164,8 +162,8 @@ main  = runC2HS (version, copyright, disclaimer) compile
 -- header is output in case of help, before the descriptions of the options;
 -- errTrailer is output after an error message
 --
-header :: String -> String -> String -> String
-header version copyright disclaimer  = 
+header :: String
+header =
   version ++ "\n" ++ copyright ++ "\n" ++ disclaimer
   ++ "\n\nUsage: c2hs [ option... ] [header-file] binding-file\n"
 
@@ -184,7 +182,6 @@ errTrailer = "Try the option `--help' on its own for more information.\n"
 --
 data Flag = CPPOpts  String     -- additional options for C preprocessor
 	  | CPP      String     -- program name of C preprocessor
-	  | Data     String     -- Data directory
 	  | Dump     DumpType   -- dump internal information
 	  | Help	        -- print brief usage information
 	  | Keep	        -- keep the .i file
@@ -193,7 +190,8 @@ data Flag = CPPOpts  String     -- additional options for C preprocessor
 	  | Output   String     -- file where the generated file should go
 	  | Platform String     -- target platform to generate code for
 	  | OutDir   String     -- directory where generates files should go
-	  | Version	        -- print version information on stderr
+	  | Version	        -- print version information on stdout
+	  | NumericVersion      -- print numeric version on stdout
 	  | Error    String     -- error occured during processing of options
 	  deriving Eq
 
@@ -215,10 +213,6 @@ options  = [
 	 ["cpp"] 
 	 (ReqArg CPP "CPP") 
 	 "use executable CPP to invoke C preprocessor",
-  Option [] 
-	 ["data"] 
-	 (ReqArg Data "DIR") 
-	 "data directory (set by wrapper script)",
   Option ['d'] 
 	 ["dump"] 
 	 (ReqArg dumpArg "TYPE") 
@@ -254,7 +248,11 @@ options  = [
   Option ['v'] 
 	 ["version"] 
 	 (NoArg Version) 
-	 "show version information"]
+	 "show version information",
+  Option []
+	 ["numeric-version"]
+	 (NoArg NumericVersion)
+	 "show version number"]
 
 -- convert argument of `Dump' option
 --
@@ -276,11 +274,11 @@ compile  =
     setup
     cmdLine <- getArgsCIO
     case getOpt RequireOrder options cmdLine of
-      (opts     , args, []) 
-        | noCompOpts opts &&
-	  null args       -> doExecute opts []
-        | properArgs args -> doExecute opts args
-        | otherwise       -> raiseErrs [wrongNoOfArgsErr]
+      (opts, []  , [])
+        | noCompOpts opts -> doExecute opts Nothing
+      (opts, args, [])    -> case parseArgs args of
+        args@(Just _)     -> doExecute opts args
+        Nothing           -> raiseErrs [wrongNoOfArgsErr]
       (_   , _   , errs)  -> raiseErrs errs
   where
     -- These options can be used without specifying a binding module.  Then,
@@ -288,21 +286,26 @@ compile  =
     -- place.  (There can be --data and --output-dir (-t) options in addition
     -- to the action.)
     --
-    aloneOptions = [Help, Version, Library]
+    aloneOptions = [Help, Version, NumericVersion, Library]
     --
     noCompOpts opts = let nonDataOpts = filter nonDataOrDir opts
 		      in
 		      (not . null) nonDataOpts &&
 		      all (`elem` aloneOptions) nonDataOpts
       where
-        nonDataOrDir (Data   _) = False
         nonDataOrDir (OutDir _) = False
 	nonDataOrDir _	        = True
     --
-    properArgs [bnd]         = suffix bnd == chssuffix 
-    properArgs [header, bnd] = suffix header == hsuffix 
-			       && suffix bnd == chssuffix 
-    properArgs _             = False
+    parseArgs :: [FilePath] -> Maybe (FilePath, [FilePath])
+    parseArgs = parseArgs' [] Nothing
+      where parseArgs' hs (Just chs) []    = Just (chs, reverse hs)
+            parseArgs' hs chs@Nothing (file:files)
+                | FilePath.takeExtension file == '.':chssuffix
+                                           = parseArgs' hs (Just file) files
+            parseArgs' hs chs (file:files)
+                | FilePath.takeExtension file == '.':hsuffix
+                                           = parseArgs' (file:hs) chs files
+            parseArgs' _  _   _            = Nothing
     --
     doExecute opts args = do
 			    execute opts args
@@ -310,8 +313,8 @@ compile  =
 			    exitWithCIO ExitSuccess
     --
     wrongNoOfArgsErr = 
-      "There must be exactly one binding file (suffix .chs), possibly\n\
-      \preceded by one header file (suffix .h).\n"
+      "There must be exactly one binding file (suffix .chs),\n\
+      \and optionally one or more header files (suffix .h).\n"
     --
     -- exception handler
     --
@@ -329,7 +332,6 @@ compile  =
 setup :: CST s ()
 setup  = do
 	   setCPP     cpp
-	   setData    datadir
 	   addCPPOpts cppopts
 
 -- output error message
@@ -350,24 +352,20 @@ raiseErrs errs = do
 -- * actual compilation is only invoked if we have one or two extra arguments
 --   (otherwise, it is just skipped)
 --
-execute :: [Flag] -> [FilePath] -> CST s ()
+execute :: [Flag] -> Maybe (FilePath, [FilePath]) -> CST s ()
 execute opts args | Help `elem` opts = help
 		  | otherwise	     = 
   do
-    let vs      = filter (== Version) opts
-	opts'   = filter (/= Version) opts
+    let (vs,opts') = partition (\opt -> opt == Version
+                                     || opt == NumericVersion) opts
     mapM_ processOpt (atMostOne vs ++ opts')
-    if length args > 0 
-      then
-	let (headerFile, bndFile) = case args of
-				      [       bfile] -> (""   , bfile)
-				      [hfile, bfile] -> (hfile, bfile)
-	    bndFileWithoutSuffix  = stripSuffix bndFile
-	in do
+    case args of
+      Just (bndFile, headerFiles) -> do
+	let bndFileWithoutSuffix  = FilePath.dropExtension bndFile
 	computeOutputName bndFileWithoutSuffix
-	process headerFile bndFileWithoutSuffix
+	process headerFiles bndFileWithoutSuffix
 	  `fatalsHandledBy` die
-      else
+      Nothing ->
 	computeOutputName "."	-- we need the output name for library copying
     copyLibrary
       `fatalsHandledBy` die
@@ -385,8 +383,7 @@ execute opts args | Help `elem` opts = help
 help :: CST s ()
 help = 
   do
-    (version, copyright, disclaimer) <- getId
-    putStrCIO (usageInfo (header version copyright disclaimer) options)
+    putStrCIO (usageInfo header options)
     putStrCIO trailer
     putStrCIO $ "PLATFORM can be " ++ hosts ++ "\n"
     putStrCIO $ "  (default is " ++ identPS defaultPlatformSpec ++ ")\n"
@@ -400,7 +397,6 @@ help =
 processOpt :: Flag -> CST s ()
 processOpt (CPPOpts  cppopts) = addCPPOpts  cppopts
 processOpt (CPP      cpp    ) = setCPP      cpp
-processOpt (Data     dir    ) = setData     dir
 processOpt (Dump     dt     ) = setDump     dt
 processOpt (Keep            ) = setKeep
 processOpt (Library         ) = setLibrary
@@ -409,11 +405,11 @@ processOpt (Output   fname  ) = setOutput   fname
 processOpt (Platform fname  ) = setPlatform fname
 processOpt (OutDir   fname  ) = setOutDir   fname
 processOpt Version            = do
-			          (version, _, _) <- getId 
 			          putStrLnCIO version
 				  platform <- getSwitch platformSB
 				  putStrCIO "  build platform is "
 				  printCIO platform
+processOpt NumericVersion     = putStrLnCIO (showVersion versnum)
 processOpt (Error    msg    ) = abort msg
 
 -- emit error message and raise an error
@@ -431,18 +427,8 @@ abort msg  = do
 --
 computeOutputName :: FilePath -> CST s ()
 computeOutputName bndFileNoSuffix =
-  do
-    output <- getSwitch outputSB
-    outDir <- getSwitch outDirSB
-    let dir  = if      null outDir && null output then dirname bndFileNoSuffix
-	       else if null outDir		  then dirname output
-	       else				       outDir
-    let base = if null output then basename bndFileNoSuffix
-	       else                basename output
-    setSwitch $ \sb -> sb {
-		         outputSB = dir `addPath` base,
-			 outDirSB = dir
-		       }
+  setSwitch $ \sb@SwitchBoard{ outputSB = output } ->
+    sb { outputSB = if null output then bndFileNoSuffix else output }
 
 -- Copy the C2HS library if requested
 --
@@ -450,9 +436,9 @@ copyLibrary =
   do
     outdir  <- getSwitch outDirSB
     library <- getSwitch librarySB
-    datadir <- getSwitch dataSB
-    let libFullName = datadir `addPath` libfname
-	libDestName = outdir  `addPath` libfname
+    datadir <- liftIO getDataDir
+    let libFullName = datadir </> libfname
+	libDestName = outdir  </> libfname
     when library $
       readFileCIO libFullName >>= writeFileCIO libDestName
 
@@ -476,11 +462,6 @@ addCPPOpts opts  =
 setCPP       :: FilePath -> CST s ()
 setCPP fname  = setSwitch $ \sb -> sb {cppSB = fname}
 
--- set the data directory
---
-setData :: FilePath -> CST s ()
-setData dir = setSwitch $ \sb -> sb {dataSB = dir}
-
 -- set the given dump option
 --
 setDump         :: DumpType -> CST s ()
@@ -524,9 +505,9 @@ setInclude str = do
 --
 setOutput       :: FilePath -> CST s ()
 setOutput fname  = do
-		     when (suffix fname /= hssuffix) $
+		     when (FilePath.takeExtension fname /= '.':hssuffix) $
 		       raiseErrs ["Output file should end in .hs!\n"]
-		     setSwitch $ \sb -> sb {outputSB = stripSuffix fname}
+		     setSwitch $ \sb -> sb {outputSB = FilePath.dropExtension fname}
 
 -- set platform
 --
@@ -560,8 +541,8 @@ setHeader fname  = setSwitch $ \sb -> sb {headerSB = fname}
 --
 -- * the binding file name has been stripped of the .chs suffix
 --
-process                    :: FilePath -> FilePath -> CST s ()
-process headerFile bndFile  =
+process                    :: [FilePath] -> FilePath -> CST s ()
+process headerFiles bndFile  =
   do
     -- load the Haskell binding module
     --
@@ -580,14 +561,23 @@ process headerFile bndFile  =
     -- CPP and inline-C of .chs file into the new header
     --
     outFName <- getSwitch outputSB
-    let newHeaderFile = outFName ++ hsuffix
-	preprocFile   = basename newHeaderFile ++ isuffix
-    newHeader <- openFileCIO newHeaderFile WriteMode
-    unless (null headerFile) $
-      hPutStrLnCIO newHeader $ "#include \"" ++ headerFile ++ "\""
-    mapM (hPutStrCIO newHeader) header
-    hCloseCIO newHeader
-    setHeader newHeaderFile
+    outDir   <- getSwitch outDirSB
+    let newHeader     = outFName <.> chssuffix <.> hsuffix
+        newHeaderFile = outDir </> newHeader
+	preprocFile   = FilePath.takeBaseName outFName <.> isuffix
+    writeFileCIO newHeaderFile $ concat $
+      [ "#include \"" ++ headerFile ++ "\"\n"
+      | headerFile <- headerFiles ]
+      ++ header
+    --
+    -- Check if we can get away without having to keep a separate .chs.h file
+    --
+    case headerFiles of
+      [headerFile] | null header
+        -> setHeader headerFile    -- the generated .hs file will directly
+                                   -- refer to this header rather than going
+                                   -- through a one-line .chs.h file.
+      _ -> setHeader newHeader
     --
     -- run C preprocessor over the header
     --
@@ -598,18 +588,23 @@ process headerFile bndFile  =
     exitCode <- systemCIO cmd
     case exitCode of 
       ExitFailure _ -> fatal "Error during preprocessing custom header file"
-      _		    -> nop
+      _		    -> return ()
     --
     -- load and analyse the C header file
     --
     (cheader, warnmsgs) <- loadAttrC preprocFile
     putStrCIO warnmsgs
     --
-    -- remove the custom header and the pre-processed header
+    -- remove the pre-processed header and if we no longer need it, remove the
+    -- custom header file too.
     --
     keep <- getSwitch keepSB
-    unless keep $
+    unless keep $ do
       removeFileCIO preprocFile
+      case headerFiles of
+        [headerFile] | null header
+          -> removeFileCIO newHeaderFile
+        _ -> return () -- keep it since we'll need it to compile the .hs file
     --
     -- expand binding hooks into plain Haskell
     --
@@ -618,8 +613,8 @@ process headerFile bndFile  =
     --
     -- output the result
     --
-    dumpCHS outFName hsMod True
-    dumpCHI outFName chi		-- different suffix will be appended
+    dumpCHS (outDir </> outFName) hsMod True
+    dumpCHI (outDir </> outFName) chi		-- different suffix will be appended
   where
     tracePreproc cmd = putTraceStr tracePhasesSW $
 		         "Invoking cpp as `" ++ cmd ++ "'...\n"
@@ -631,4 +626,4 @@ process headerFile bndFile  =
 					 ++ "'...\n")
 			      dumpCHS chsName mod False)
 
-    chsName = basename bndFile ++ ".dump"
+    chsName = FilePath.takeBaseName bndFile <.> "dump"
